#-------------------------------------------------------------------------------
# controlflow_analyzer.py
#
# Controlflow analyzer 
#
# for visualization, graphviz and pygraphviz are required
# (only Python 2.7 is supported)
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

import pyverilog.utils.version
import pyverilog.utils.util as util
import pyverilog.utils.signaltype as signaltype
import pyverilog.utils.inference as inference
import pyverilog.dataflow.reorder as reorder
import pyverilog.dataflow.replace as replace
from pyverilog.dataflow.subset import VerilogSubset
from pyverilog.dataflow.walker import VerilogDataflowWalker
from pyverilog.dataflow.dataflow import *

if sys.version_info[0] >= 3:
    import pyverilog.controlflow.splitter as splitter
    import pyverilog.controlflow.transition as transition
else:
    import splitter
    import transition

class VerilogControlflowAnalyzer(VerilogSubset):
    def __init__(self, topmodule, terms, binddict,
                 resolved_terms, resolved_binddict, 
                 constlist, fsm_vars=('fsm', 'state', 'count', 'cnt', 'step', 'mode') ):
        VerilogSubset.__init__(self, topmodule, terms, binddict,
                               resolved_terms, resolved_binddict, constlist)
        self.treewalker = VerilogDataflowWalker(topmodule, terms, binddict, 
                                                resolved_terms, resolved_binddict, constlist)
        self.fsm_vars = fsm_vars

    ############################################################################
    def getLoops(self):
        fsms = self.getFiniteStateMachines()
        loops = {}
        for signame, fsm in fsms.items():
            loop_set = fsm.get_loop()
            if len(loop_set) == 0: continue
            if not signame in loops: loops[signame] = set([])
            loops[signame].update(loop_set)
        return loops, fsms

    def getFiniteStateMachines(self):
        statemachines = {}
        for termname, bindlist in self.resolved_binddict.items():
            if not self.isFsmVar(termname): continue
            funcdict, delaycnt = self.getFuncdict(termname)
            if len(funcdict) > 0: print("FSM signal: %s, Condition list length: %d" % (str(termname), len(funcdict)))
            fsm = self.getFiniteStateMachine(termname, funcdict)
            if fsm.size() > 0:
                fsm.set_delaycnt(delaycnt)
                fsm.resolve(self.optimizer)
                statemachines[termname] = fsm
        return statemachines

    ############################################################################
    def getFiniteStateMachine(self, termname, funcdict):
        fsm = FiniteStateMachine(util.toFlatname(termname))
        if len(funcdict) == 0: return fsm
        width = self.getWidth(termname)
        for condlist, func in sorted(funcdict.items(), key=lambda x:len(x[0])):
            if not isinstance(func, DFEvalValue): continue
            print("Condition: %s, Inferring transition condition" % str(condlist))
            node = transition.walkCondlist(condlist, termname, width)
            if node is None: continue
            statenode_list = node.nodelist if isinstance(node, transition.StateNodeList) else [node,]
            for statenode in statenode_list: fsm.construct(func.value, statenode)
        return fsm

    def getFuncdict(self, termname, delaycnt=0):
        termtype = self.getTermtype(termname)
        if not self.isClockEdge(termname): return {}, 0
        if signaltype.isRename(termtype): return {}, 0
        if signaltype.isRegArray(termtype): return {}, 0 # currently unsupported
        tree = self.makeTree(termname)
        funcdict = splitter.split(tree)
        funcdict = splitter.remove_reset_condition(funcdict)
        if len(funcdict) == 1 and len(funcdict.keys()[0]) == 0:
            next_term = funcdict.values()[0]
            if isinstance(next_term, DFTerminal):
                return self.getFuncdict(next_term.name, delaycnt+1)
        return funcdict, delaycnt

    def isFsmVar(self, termname):
        for v in self.fsm_vars:
            if re.search(v.lower(), str(termname).lower()): return True
        return False

    def getWidth(self, termname):
        term = self.getTerm(termname)
        msb = self.optimizer.optimizeConstant(term.msb)
        lsb = self.optimizer.optimizeConstant(term.lsb)
        width = DFIntConst('32')
        if msb is not None and lsb is not None:
            return abs(self.optimizer.optimizeConstant(DFOperator((msb, lsb), 'Minus')).value) + 1
        return self.optimizer.optimizeConstant(width).value

    def makeTree(self, termname):
        tree = self.getTree(termname)
        tree = self.treewalker.walkTree(tree)
        tree = reorder.reorder(tree)
        tree = self.optimizer.optimize(tree)
        tree = replace.replaceUndefined(tree, termname)
        return tree

################################################################################
class FiniteStateMachine(object):
    def __init__(self, name):
        self.name = name
        self.fsm = {} # key:src, value: dict[cond]=dst
        self.any = {} # key:cond, value:dst
        self.delaycnt = 0

    def set_delaycnt(self, delaycnt):
        self.delaycnt = delaycnt

    def size(self):
        dstlen = 0
        for src, dstdict in self.fsm.items():
            dstlen += len(dstdict)
        srcs = self.fsm.keys()
        dstlen += len(srcs) * len(self.any)
        return dstlen

    def label_range(self):
        minval = None
        maxval = None
        for src, dstdict in self.fsm.items():
            if minval is None: minval = src
            elif src < minval: minval = src
            if maxval is None: maxval = src
            elif src > maxval: maxval = src
            for cond, dst in dstdict.items():
                if minval is None: minval = dst
                elif dst < minval: minval = dst
                if maxval is None: maxval = dst
                elif dst > maxval: maxval = dst
        return (minval, maxval)

    def construct(self, dst, node):
        if node is None: return
        if node.isany:
            transcond = node.transcond
            self.add_any(dst, transcond)
        for rp in node.range_pairs:
            transcond = node.transcond
            self.add(rp, dst, transcond)
        
    def add_any(self, dst, cond):
        self.any[cond] = dst

    def add(self, srcs, dst, cond):
        sb, se = srcs
        for src in range(sb, se+1):
            if not src in self.fsm:
                self.fsm[src] = {}
            self.fsm[src][cond] = dst
    
    def resolve(self, evaluate):
        new_fsm = {}
        for src, dstdict in sorted(self.fsm.items(), key=lambda x:x[0]):
            dst_cond_dict = {}
            for cond, dst in sorted(dstdict.items(), key=lambda x:x[1]):
                if not dst in dst_cond_dict:
                    dst_cond_dict[dst] = cond
                else:
                    cur_cond = dst_cond_dict[dst]
                    if cur_cond is None: pass
                    elif cond is None: dst_cond_dict[dst] = None
                    else: dst_cond_dict[dst] = evaluate.optimize(DFOperator((cur_cond, cond), 'Lor'))
            new_dstdict = {}
            for dst, cond in dst_cond_dict.items():
                if isinstance(cond, DFEvalValue) and cond.value > 0:
                    new_dstdict[None] = dst
                else:
                    new_dstdict[cond] = dst
            new_fsm[src] = new_dstdict
        self.fsm = new_fsm
                        
    def view(self):
        for cond, dst in self.any.items():
            s = []
            s.append('any -- ')
            if cond is not None: s.append(cond.tocode())
            else: s.append('None')
            s.append('--> %d' % dst)
            print(''.join(s))
        for src, dstdict in self.fsm.items():
            for cond, dst in dstdict.items():
                s = []
                s.append('%d --' % src)
                if cond is not None: s.append(cond.tocode())
                else: s.append('None')
                s.append('--> %d' % dst)
                print(''.join(s))

    def tograph(self, filename='fsm.png', nolabel=False):
        import pygraphviz as pgv # Python 3 currently does not support Pygraphviz

        graph = pgv.AGraph(directed=True) #pgv.AGraph(strict=False, directed=True)
        for src, dstdict in self.fsm.items():
            graph.add_node(str(src), label=str(src))
            for cond, dst in dstdict.items():
                graph.add_node(str(dst), label=str(dst))
                if nolabel:
                    graph.add_edge(str(src), str(dst), label='')
                else:
                    graph.add_edge(str(src), str(dst), label=str(cond))
        srcs = self.fsm.keys()
        for src in srcs:
            for cond, dst in self.any.items():
                graph.add_node(str(dst), label=str(dst))
                if nolabel:
                    graph.add_edge(str(src), str(dst), label='')
                else:
                    graph.add_edge(str(src), str(dst), label=str(cond))

        graph.write('file.dot')
        graph.layout(prog='dot')
        graph.draw(filename)

    ############################################################################
    def get_loop(self):
        loops = set([])
        loop_node_cnt = {}
        for k in sorted(self.fsm.keys()):
            if not k in self.fsm:
                continue
            if not k in loop_node_cnt: loop_node_cnt[k] = 0
            if loop_node_cnt[k] >= len(self.fsm[k].values()):
                continue
            paths = self.get_looppath(k)
            for path in paths:
                rotated = self.rotate(path)
                if rotated not in loops:
                    loops.add(rotated)
                    for r in rotated:
                        if not r in loop_node_cnt: loop_node_cnt[r] = 0
                        loop_node_cnt[r] += 1
        return loops

    def rotate(self, path):
        minval = min(path)
        minval_pos = path.index(minval)
        return path[minval_pos:] + path[:minval_pos]

    def get_looppath(self, src):
        paths = set([])
        if not src in self.fsm: return paths
        nextnodes = set(self.fsm[src].values())
        for n in nextnodes:
            r = self._looppath(src, n, visited=set([n]))
            if len(r) > 0: paths |= r
        return paths

    def _looppath(self, src, node, visited, cnt=0):
        paths = set([])
        if cnt > 50:
            return paths
        if src == node:
            paths.add( (src,) )
            return paths
        if not node in self.fsm:
            return paths
        nextnodes = set(self.fsm[node].values()) - visited
        for n in nextnodes:
            r = self._looppath(src, n, visited=visited|set([node]), cnt=cnt+1)
            if len(r) > 0:
                for np in r: paths.add( (node,) + np )
        return paths

################################################################################
if __name__ == '__main__':
    from optparse import OptionParser
    import pyverilog.utils.util as util
    from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
    from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
    INFO = "Control-flow analyzer for Verilog definitions"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python controlflow_analyzer.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    optparser.add_option("--graphformat",dest="graphformat",
                         default="png",help="Graph file format, Default=png")
    optparser.add_option("--nograph",action="store_true",dest="nograph",
                         default=False,help="Non graph generation")
    optparser.add_option("--nolabel",action="store_true",dest="nolabel",
                         default=False,help="State Machine Graph without Labels")
    optparser.add_option("-I","--include",dest="include",action="append",
                         default=[],help="Include path")
    optparser.add_option("-D",dest="define",action="append",
                         default=[],help="Macro Definition")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule,
                                       preprocess_include=options.include,
                                       preprocess_define=options.define)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)

    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()
    fsm_vars = tuple(['fsm', 'state', 'count', 'cnt', 'step', 'mode'] + options.searchtarget)
    
    canalyzer = VerilogControlflowAnalyzer(options.topmodule, terms, binddict,
                                           resolved_terms, resolved_binddict, constlist, fsm_vars)
    fsms = canalyzer.getFiniteStateMachines()

    for signame, fsm in fsms.items():
        print('# SIGNAL NAME: %s' % signame)
        print('# DELAY CNT: %d' % fsm.delaycnt)
        fsm.view()
        if not options.nograph:
            fsm.tograph(filename=util.toFlatname(signame)+'.'+options.graphformat, nolabel=options.nolabel)
        loops = fsm.get_loop()
        print('Loop')
        for loop in loops:
            print(loop)



#-----------------------------------------------------------------
# plyparser.py
#
# PLYParser class and other utilites for simplifying programming
# parsers with PLY
#
# Copyright (C) 2008-2011, Eli Bendersky
# License: BSD
#-----------------------------------------------------------------


class Coord(object):
    """ Coordinates of a syntactic element. Consists of:
            - File name
            - Line number
            - (optional) column number, for the Lexer
    """
    def __init__(self, file, line, column=None):
        self.file = file
        self.line = line
        self.column = column

    def __str__(self):
        str = "%s:%s" % (self.file, self.line)
        if self.column: str += ":%s" % self.column
        return str


class ParseError(Exception): pass


class PLYParser(object):
    def _create_opt_rule(self, rulename):
        """ Given a rule name, creates an optional ply.yacc rule
            for it. The name of the optional rule is
            <rulename>_opt
        """
        optname = rulename + '_opt'

        def optrule(self, p):
            p[0] = p[1]

        optrule.__doc__ = '%s : empty\n| %s' % (optname, rulename)
        optrule.__name__ = 'p_%s' % optname
        setattr(self.__class__, optrule.__name__, optrule)
    
    def _coord(self, lineno, column=None):
        return Coord(
                file=self.lexer.filename, 
                line=lineno,
                column=column)

    def _parse_error(self, msg, coord):
        raise ParseError("%s: %s" % (coord, msg))



#-------------------------------------------------------------------------------
# active_range.py (Obsoluted)
#
# Active condition analyzer from dataflow
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

import pyverilog.utils.version
import pyverilog.utils.util as util
import pyverilog.utils.signaltype as signaltype
import pyverilog.utils.inference as inference
import pyverilog.dataflow.reorder as reorder
from pyverilog.dataflow.dataflow import *

if sys.version_info[0] >= 3:
    import pyverilog.controlflow.splitter as splitter
    from pyverilog.controlflow.controlflow_analyzer import VerilogControlflowAnalyzer
else:
    import splitter
    from controlflow_analyzer import VerilogControlflowAnalyzer

class VerilogActiveAnalyzer(VerilogControlflowAnalyzer):
    def __init__(self, topmodule, terms, binddict, 
                 resolved_terms, resolved_binddict, constlist):
        VerilogControlflowAnalyzer.__init__(self, topmodule, terms, binddict, 
                                            resolved_terms, resolved_binddict, constlist)

    ############################################################################
    def getActiveConditions(self, termname, op='>', conditionvalue=0):
        if not termname in self.resolved_binddict: return ()
        tree = self.makeConditionalTree(termname, step=0)
        funcdict = splitter.split(tree)
        condlists = self.getActiveFuncdictKeys(funcdict, op=op, conditionvalue=conditionvalue)
        active_conditions = self.inferActiveConditions(condlists)
        return active_conditions # OR-style

    ############################################################################
    def getChangedConditions(self, termname):
        if not termname in self.resolved_binddict: return ()
        bindlist = self.resolved_binddict[termname]
        termtype = self.getTermtype(termname)
        tree = self.makeTree(termname)
        funcdict = splitter.split(tree)
        if len(funcdict) == 0: return ()
        changed_condlists = self.getChangedFuncdictKeys(funcdict, termname)
        changed_conditions = self.inferActiveConditions(changed_condlists)
        return changed_conditions

    def getChangedConditionsWithAssignments(self, termname):
        if not termname in self.resolved_binddict: return {}
        bindlist = self.resolved_binddict[termname]
        termtype = self.getTermtype(termname)
        tree = self.makeTree(termname)
        funcdict = splitter.split(tree)
        if len(funcdict) == 0: return {}
        changed_condfuncs = self.getChangedFuncdict(funcdict, termname)
        changed_conditiondict = self.inferActiveConditionDict(changed_condfuncs)
        return changed_conditiondict

    ############################################################################
    def getUnchangedConditions(self, termname):
        if not termname in self.resolved_binddict: return ()
        bindlist = self.resolved_binddict[termname]
        termtype = self.getTermtype(termname)
        tree = self.makeTree(termname)
        funcdict = splitter.split(tree)
        if len(funcdict) == 0: return ()
        unchanged_condlists = self.getUnchangedFuncdictKeys(funcdict, termname)
        unchanged_conditions = self.inferActiveConditions(unchanged_condlists)
        return unchanged_conditions

    ############################################################################
    def makeConditionalTree(self, termname, step=0):
        tree = self.makeTree(termname)
        return tree

    ############################################################################
    def getActiveFuncdictKeys(self, funcdict, op='>', conditionvalue=0):
        activekeys = []
        for _condlist, func in funcdict.items():
            condlist = splitter.remove_reset_condlist(_condlist)
            if isinstance(func, DFEvalValue):
                e = eval('func.value'+op+str(conditionvalue))
                if e: activekeys.append(condlist)
        return activekeys

    def getChangedFuncdictKeys(self, funcdict, termname):
        changedkeys = []
        for _condlist, func in funcdict.items():
            condlist = splitter.remove_reset_condlist(_condlist)
            if not (isinstance(func, DFTerminal) and func.name == termname):
                changedkeys.append(condlist)
        return changedkeys

    def getUnchangedFuncdictKeys(self, funcdict, termname):
        unchangedkeys = []
        for _condlist, func in funcdict.items():
            condlist = splitter.remove_reset_condlist(_condlist)
            if isinstance(func, DFTerminal) and func.name == termname:
                unchangedkeys.append(condlist)
        return unchangedkeys

    def getChangedFuncdict(self, funcdict, termname):
        changeddict = {}
        for _condlist, func in funcdict.items():
            condlist = splitter.remove_reset_condlist(_condlist)
            if not (isinstance(func, DFTerminal) and func.name == termname):
                changeddict[condlist] = func
        return changeddict

    ############################################################################
    def inferActiveConditions(self, condlists):
        active_conditions = []
        for condlist in condlists:
            l = self.inferActiveCondition(condlist)
            if isinstance(l, ActiveTerm):
                activecond = ActiveCondition()
                activecond.set(l.name, l)
                if not (activecond in active_conditions):
                    active_conditions.append(activecond)
            if isinstance(l, ActiveCondition):
                l.resolve()
                if len(l) > 0 and not (l in active_conditions):
                    active_conditions.append(l)
            if isinstance(l, ActiveConditionList):
                for condition in l.conditions:
                    condition.resolve()
                    if not (condition in active_conditions):
                        active_conditions.append(condition)
        return tuple(active_conditions)
        
    def inferActiveConditionDict(self, condfuncs):
        active_conditions = {}
        for condlist, func in condfuncs.items():
            l = self.inferActiveCondition(condlist)
            if isinstance(l, ActiveTerm):
                if len(l.range_pairs) == 0: continue
                activecond = ActiveCondition()
                activecond.set(l.name, l)
                if not (activecond in active_conditions):
                    active_conditions[activecond] = func
            if isinstance(l, ActiveCondition):
                l.resolve()
                if len(l) > 0 and not (l in active_conditions):
                    active_conditions[l] = func
            if isinstance(l, ActiveConditionList):
                for condition in l.conditions:
                    condition.resolve()
                    if not (condition in active_conditions):
                        active_conditions[condition] = func
        return active_conditions
        
    ############################################################################
    def inferActiveCondition(self, condlist):
        condition = None
        for cond in condlist:
            walkrslt = self.walkActiveCond(cond)
            if condition is None: condition = walkrslt
            else: condition = self._activecond_opAnd(condition, walkrslt)
        return condition

    ############################################################################        
    def walkActiveCond(self, cond):
        if isinstance(cond, DFTerminal):
            return self._walkActiveCond(DFOperator((cond, DFEvalValue(0)),'GreaterThan'))
        return self._walkActiveCond(cond)

    def _walkActiveCond(self, cond):
        if isinstance(cond, DFOperator):
            if len(cond.nextnodes) == 1:
                return self._walkActiveCond_DFOperator_unary(cond)
            if len(cond.nextnodes) == 2:
                return self._walkActiveCond_DFOperator_dual(cond)
        return None

    def _walkActiveCond_DFOperator_unary(self, cond):
        activecond = self.walkActiveCond(cond.nextnodes[0])
        if signaltype.isNot(cond.operator): return self._activecond_opNot(activecond)
        return None

    def _walkActiveCond_DFOperator_dual(self, cond):
        l = cond.nextnodes[0]
        r = cond.nextnodes[1]

        if signaltype.isOr(cond.operator):
            lactivecond = self.walkActiveCond(l)
            ractivecond = self.walkActiveCond(r)
            if lactivecond is None and ractivecond is not None: return ractivecond
            if lactivecond is not None and ractivecond is None: return lactivecond
            return self._activecond_opOr(lactivecond, ractivecond)

        if signaltype.isAnd(cond.operator):
            lactivecond = self.walkActiveCond(l)
            ractivecond = self.walkActiveCond(r)
            if lactivecond is None and ractivecond is not None: return ractivecond
            if lactivecond is not None and ractivecond is None: return lactivecond
            return self._activecond_opAnd(lactivecond, ractivecond)

        if isinstance(l, DFTerminal):
            infval = inference.infer(cond.operator, r)
            minval = 0
            maxval = util.maxValue(self.getWidth(l.name))
            range_pairs = ((infval.minval, infval.maxval),)
            return ActiveTerm(l.name, range_pairs, minval, maxval)

        if isinstance(r, DFTerminal):
            new_op = re.sub('Greater', 'TMP', cond.operator)
            new_op = re.sub('Less', 'Greater', new_op)
            new_op = re.sub('TMP', 'Less', new_op)
            infval = inference.infer(new_op, l)
            minval = 0
            maxval = util.maxValue(self.getWidth(r.name))
            range_pairs = ((infval.minval, infval.maxval),)
            return ActiveTerm(r.name, range_pairs, minval, maxval)

        if cond.operator == 'Eq':
            lactivecond = self.walkActiveCond(l)
            ractivecond = self.walkActiveCond(r)
            if lactivecond is not None and ractivecond is None and isinstance(r, DFEvalValue):
                if r.value > 0: return lactivecond
                return self._activecond_opNot(lactivecond)
            if lactivecond is None and ractivecond is not None and isinstance(l, DFEvalValue):
                if l.value > 0: return ractivecond
                return self._activecond_opNot(ractivecond)
            return self._activecond_opAnd(lactivecond, ractivecond)

        if cond.operator == 'NotEq':
            lactivecond = self.walkActiveCond(l)
            ractivecond = self.walkActiveCond(r)
            if lactivecond is not None and ractivecond is None and isinstance(r, DFEvalValue):
                if r.value == 0: return lactivecond
                return self._activecond_opNot(lactivecond)
            if lactivecond is None and ractivecond is not None and isinstance(l, DFEvalValue):
                if l.value == 0: return ractivecond
                return self._activecond_opNot(ractivecond)
            return None
        return None

    def _activecond_opNot(self, node):
        if isinstance(node, ActiveTerm):
            new_node = node
            new_node.opNot()
            return new_node
        if isinstance(node, ActiveCondition):
            activelist = []
            for name, term in node.termdict.items():
                new_term = term
                new_term.opNot()
                new_cond = ActiveCondition()
                new_cond.set(name, new_term)
                activelist.append(new_cond)
            return ActiveConditionList(tuple(activelist))
        if isinstance(node, ActiveConditionList):
            activelist = []
            now_appended = []
            for cnt in range(len(node.conditions)):
                if cnt == 0: continue
                left = node.conditions[cnt-1]
                right = node.conditions[cnt]
                if cnt == 1:                    
                    for lname, lterm in left.termdict.items():
                        not_lterm = lterm.opNot()
                        for rname, rterm in right.termdict.items():
                            not_rterm = rterm.opNot()
                            r = self._activecond_opAnd(not_lterm, not_rterm)
                            if r: now_appended.append(r)
                    activelist.extend(now_appended)
                else:
                    left = ActiveConditionList(tuple(now_appended))
                    now_appended = []
                    for lcondition in left.conditions:
                        for rname, rterm in right.termdict.items():
                            not_rterm = rterm.opNot()
                            r = self._activecond_opAnd(lcondition, not_rterm)
                            if r: now_appended.append(r)
                    activelist.extend(now_appended)
            if len(activelist) > 0:
                return ActiveConditionList(tuple(activelist))
        return None

    def _activecond_opAnd(self, left, right):
        if isinstance(left, ActiveTerm):
            return self._activecond_opAnd_right_term(left, right)
        if isinstance(left, ActiveCondition):
            return self._activecond_opAnd_right_condition(left, right)
        if isinstance(left, ActiveConditionList):
            return self._activecond_opAnd_right_conditionlist(left, right)
        return None

    def _activecond_opAnd_right_term(self, left, right):
        if isinstance(right, ActiveTerm):
            new_cond = ActiveCondition()
            new_cond.set(left.name, left)
            new_cond.set(right.name, right)
            return new_cond
        if isinstance(right, ActiveCondition):
            new_cond = ActiveCondition()
            new_cond.set(left.name, left)
            for name, term in right.termdict.items():
                new_cond.set(name, term)
            return new_cond
        if isinstance(right, ActiveConditionList):
            activelist = []
            for condition in right.conditions:
                new_condition = condition
                new_condition.set(left.name, left)
                activelist.append(new_condition)
            return ActiveConditionList(tuple(activelist))
        return None

    def _activecond_opAnd_right_condition(self, left, right):
        if isinstance(right, ActiveTerm):
            new_cond = ActiveCondition()
            for name, term in left.termdict.items():
                new_cond.set(name, term)
            new_cond.set(right.name, right)
            return new_cond
        if isinstance(right, ActiveCondition):
            new_cond = ActiveCondition()
            for name, term in left.termdict.items():
                new_cond.set(name, term)
            for name, term in right.termdict.items():
                new_cond.set(name, term)
            return new_cond
        if isinstance(right, ActiveConditionList):
            activelist = []
            for condition in right.conditions:
                new_condition = self._activecond_opAnd_right_condition(left, condition)
                activelist.append(new_condition)
            return ActiveConditionList(tuple(activelist))
        return None

    def _activecond_opAnd_right_conditionlist(self, left, right):
        if isinstance(right, ActiveTerm):
            activelist = []
            for condition in left.conditions:
                new_condition = condition
                new_condition.set(right.name, right)
                activelist.append(new_condition)
            return ActiveConditionList(tuple(activelist))
        if isinstance(right, ActiveCondition):
            activelist = []
            for condition in left.conditions:
                new_condition = self._activecond_opAnd_right_condition(condition, right)
                activelist.append(new_condition)
            return ActiveConditionList(tuple(activelist))
        if isinstance(right, ActiveConditionList):
            activelist = []
            for rcondition in right.conditions:
                for condition in left.conditions:
                    new_condition = self._activecond_opAnd_right_condition(condition, rcondition)
                    activelist.append(new_condition)
            return ActiveConditionList(tuple(activelist))
        return None

    def _activecond_opOr(self, left, right):
        if isinstance(left, ActiveTerm):
            return self._activecond_opOr_right_term(left, right)
        if isinstance(left, ActiveCondition):
            return self._activecond_opOr_right_condition(left, right)
        if isinstance(left, ActiveConditionList):
            return self._activecond_opOr_right_conditionlist(left, right)
        return None

    def _activecond_opOr_right_term(self, left, right):
        if isinstance(right, ActiveTerm):
            activelist = []
            lactivecond = ActiveCondition()
            lactivecond.set(left.name, left)
            activelist.append(lactivecond)
            ractivecond = ActiveCondition()
            ractivecond.set(right.name, right)
            activelist.append(ractivecond)
            return ActiveConditionList(tuple(activelist))
        if isinstance(right, ActiveCondition):
            activelist = []
            lactivecond = ActiveCondition()
            lactivecond.set(left.name, left)
            activelist.append(lactivecond)
            activelist.append(right)
            return ActiveConditionList(tuple(activelist))
        if isinstance(right, ActiveConditionList):
            activelist = []
            lactivecond = ActiveCondition()
            lactivecond.set(left.name, left)
            activelist.append(lactivecond)
            for condition in right.conditions:
                activelist.append(condition)
            return ActiveConditionList(tuple(activelist))
        return None
        
    def _activecond_opOr_right_condition(self, left, right):
        if isinstance(right, ActiveTerm):
            activelist = []
            activelist.append(left)
            ractivecond = ActiveCondition()
            ractivecond.set(right.name, right)
            activelist.append(ractivecond)
            return ActiveConditionList(tuple(activelist))
        if isinstance(right, ActiveCondition):
            activelist = []
            activelist.append(left)
            activelist.append(right)
            return ActiveConditionList(tuple(activelist))
        if isinstance(right, ActiveConditionList):
            activelist = []
            activelist.append(left)
            for condition in right.conditions:
                activelist.append(condition)
            return ActiveConditionList(tuple(activelist))
        return None
        
    def _activecond_opOr_right_conditionlist(self, left, right):
        if isinstance(right, ActiveTerm):
            activelist = []
            for condition in left.conditions:
                activelist.append(condition)
            ractivecond = ActiveCondition()
            ractivecond.set(right.name, right)
            activelist.append(ractivecond)
            return ActiveConditionList(tuple(activelist))
        if isinstance(right, ActiveCondition):
            activelist = []
            for condition in left.conditions:
                activelist.append(condition)
            activelist.append(right)
            return ActiveConditionList(tuple(activelist))
        if isinstance(right, ActiveConditionList):
            activelist = []
            for condition in left.conditions:
                activelist.append(condition)
            for condition in right.conditions:
                activelist.append(condition)
            return ActiveConditionList(tuple(activelist))
        return None
    
################################################################################
class ActiveTerm(object):
    def __init__(self, name, range_pairs=(), minval=0, maxval=0):
        self.name = name
        self.range_pairs = range_pairs
        self.minval = minval
        self.maxval = maxval
    def set(self, range_pairs):
        self.range_pairs = range_pairs

    def __eq__(self, o):
        return self.name == o.name and self.range_pairs == o.range_pairs and self.minval == o.minval and self.maxval == o.maxval

    def opNot(self):
        if len(self.range_pairs) > 0: self._inv_range_pairs()

    def _inv_range_pairs(self):
        new_range_pairs = []
        cur_min = self.minval
        for rmin, rmax in sorted(self.range_pairs, key=lambda x:x[0]):
            if rmax is None:
                rmax = self.maxval
            if rmin-1 >= cur_min:
                new_range_pairs.append( (cur_min, rmin-1) )
            cur_min = max(rmax+1, cur_min)
        if cur_min <= self.maxval:
            new_range_pairs.append( (cur_min, self.maxval) )
        self.range_pairs = tuple(new_range_pairs)

    def __repr__(self):
        ret = '('
        ret += util.toFlatname(self.name) + ' '
        for rmin, rmax in self.range_pairs:
            ret += str(rmin) +':'+ str(rmax) + ', '
        ret = ret[:-2]
        ret += ')'
        return ret

    def tocode(self):
        ret = '('
        for rmin, rmax in self.range_pairs:
            ret += '('
            ret += str(rmin) + '<=' + util.toFlatname(self.name)
            if rmax is not None: 
                ret += '&&'
                ret += util.toFlatname(self.name) + '<=' + str(rmax)
            ret += ')'
            ret += '||'
        return ret[:-2] + ')'

    def toTree(self):
        retnode = None
        for rmin, rmax in self.range_pairs:
            if rmax is None:
                t = DFOperator((DFEvalValue(rmin), DFTerminal(self.name)), 'LessEq')
                if retnode is None: retnode = t
                else: retnode = DFOperator( (retnode, t), 'Lor')
            else:
                t = DFOperator( (DFOperator((DFEvalValue(rmin), DFTerminal(self.name)), 'LessEq'), DFOperator((DFTerminal(self.name), DFEvalValue(rmax)), 'GreaterEq')), 'Land' )
                if retnode is None: retnode = t
                else: retnode = DFOperator( (retnode, t), 'Lor')
        return retnode

    def include(self, termname):
        if termname == self.name: return True
        return False

    def satisfy(self, value):
        for rmin, rmax in sorted(self.range_pairs, key=lambda x:x[0]):
            if rmin <= value and value <= rmax: return True
        return False

class ActiveCondition(object):
    def __init__(self):
        self.termdict = {}

    def has(self, name):
        return name in self.termdict

    def set(self, name, term):
        if not self.has(name):
            self.termdict[name] = term
            return
        self._andTerm(name, term)

    def empty(self):
        return len(self.termdict) == 0

    def resolve(self):
        new_termdict = {}
        for name, term in self.termdict.items():
            if len(term.range_pairs) == 0: continue
            new_termdict[name] = term
        self.termdict = new_termdict

    def tocode(self):
        if len(self.termdict) == 0: return '1'
        ret = '('
        for name, term in self.termdict.items():
            ret += term.tocode()
            ret += '&&'
        return ret[:-2] + ')'

    def toTree(self):
        if len(self.termdict) == 0: return DFEvalValue(1)
        retnode = None
        for name, term in self.termdict.items():
            t = term.toTree()
            if retnode is None: retnode = t
            else: retnode = DFOperator( (retnode, t), 'Land')
        return retnode

    def __repr__(self):
        if len(self.termdict) == 0: return '(empty)'
        ret = '('
        for name, term in sorted(self.termdict.items(), key=lambda x:str(x[0])):
            ret += term.__repr__() + ' && '
        ret = ret[:-4] + ')'
        return ret

    def __len__(self):
        return len(self.termdict)

    def _andTerm(self, name, term):
        current_term = self.termdict[name]
        new_range_pairs = self._and_range_pairs(current_term, term)
        self.termdict[name].set(new_range_pairs)
        
    def _and_range_pairs(self, sna, snb):
        range_pairs = []
        a_ptr = 0
        b_ptr = 0
        sorted_sna = sorted(sna.range_pairs, key=lambda x:x[0])
        sorted_snb = sorted(snb.range_pairs, key=lambda x:x[0])
        if len(sorted_sna) == 0: return ()
        if len(sorted_snb) == 0: return ()
        amin, amax = sorted_sna[a_ptr]
        bmin, bmax = sorted_snb[b_ptr]
        last_a_ptr = -1
        last_b_ptr = -1
        while True:
            next_min = max(amin, bmin)
            next_max = min(amax, bmax)
            if bmax < amin or bmin > amax: pass
            else: range_pairs.append( (next_min, next_max) )
            if amax >= bmax and b_ptr < len(sorted_snb)-1:
                b_ptr += 1
                bmin, bmax = sorted_snb[b_ptr]
            elif amax <= bmax and a_ptr < len(sorted_sna)-1:
                a_ptr += 1
                amin, amax = sorted_sna[a_ptr]
            if a_ptr == last_a_ptr and b_ptr == last_b_ptr: break
            last_a_ptr = a_ptr
            last_b_ptr = b_ptr
        ret_set = set(range_pairs)
        ret_list = tuple(ret_set)
        return ret_list

    def include(self, termname):
        for name, term in self.termdict.items():
            if term.include(termname): return True
        return False

    def split(self, termname):
        termdict_including = {}
        termdict_excluding = {}
        for name, term in self.termdict.items():
            if term.include(termname): termdict_including[name] = term
            else: termdict_excluding[name] = term
        including = ActiveCondition()
        including.termdict = termdict_including
        excluding = ActiveCondition()
        excluding.termdict = termdict_excluding
        return including, excluding
    
    def satisfy(self, valuedict):
        for termname, value in valuedict.items():
            if not termname in self.termdict: continue
            if not self.termdict[termname].satisfy(value): return False
        return True

class ActiveConditionList(object):
    def __init__(self, conditions=()):
        self.conditions = conditions

    def __len__(self):
        return len(self.conditions)

    def __repr__(self):
        ret = ''
        for c in self.conditions:
            ret += c.__repr__() + ' || '
        ret = ret[:-4] + ''
        return ret

    def andterm(self, name, term):
        if len(self.conditions) == 0:
            self.conditions = (term,)
        else:
            for condition in self.conditions:
                condition.set(name, term)

    def include(self, termname):
        for condition in self.conditions:
            if condition.include(termname): return True
        return False

################################################################################
if __name__ == '__main__':
    from optparse import OptionParser
    import pyverilog.utils.util as util
    from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
    from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
    INFO = "Active condition analyzer (Obsoluted)"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python active_range.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)

    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()

    aanalyzer = VerilogActiveAnalyzer(options.topmodule, terms, binddict, 
                                      resolved_terms, resolved_binddict, constlist)

    for target in options.searchtarget:
        signal = util.toTermname(target)

        print('Active Conditions: %s' % signal)
        active_conditions = aanalyzer.getActiveConditions( signal )
        print(sorted(active_conditions, key=lambda x:str(x)))

        print('Changed Conditions')
        changed_conditions = aanalyzer.getChangedConditions( signal )
        print(sorted(changed_conditions, key=lambda x:str(x)))
    
        print('Changed Condition Dict')
        changed_conditiondict = aanalyzer.getChangedConditionsWithAssignments( signal )
        print(sorted(changed_conditiondict.items(), key=lambda x:str(x[0])))
    
        print('Unchanged Conditions')
        unchanged_conditions = aanalyzer.getUnchangedConditions( signal )
        print(sorted(unchanged_conditions, key=lambda x:str(x)))



#-------------------------------------------------------------------------------
# visit.py
# 
# Basic classes for binding tree analysis
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os
import re
import copy
import collections

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

import pyverilog
import pyverilog.utils.util as util
import pyverilog.utils.verror as verror
from pyverilog.utils.scope import ScopeLabel, ScopeChain
from pyverilog.vparser.ast import *

map_key   = lambda f,d: collections.OrderedDict([ (f(k),v) for k,v in d.items() ])
map_value = lambda f,d: collections.OrderedDict([ (k,f(v)) for k,v in d.items() ])

################################################################################
# Primitive list
################################################################################
primitives = {
    'and' : Uand,
    'nand': Unand,
    'or' : Uor,
    'nor' : Unor,
    'xor' : Uxor,
    'xnor' : Uxnor,
    'not' : Unot,
    'buf' : None
}

################################################################################
# Base Visitor
################################################################################
class NodeVisitor(object):
    def visit(self, node):
        method = 'visit_' + node.__class__.__name__
        visitor = getattr(self, method, self.generic_visit)
        return visitor(node)
    def generic_visit(self, node):
        for c in node.children():
            self.visit(c)

################################################################################
# Signal/Object Management Classes
################################################################################
class AlwaysInfo(object):
    def __init__(self, clock_name='', clock_edge=None, clock_bit=0,
                 reset_name='', reset_edge=None, reset_bit=0, senslist=()):
        self.clock_name = clock_name
        self.clock_edge = clock_edge
        self.clock_bit = clock_bit
        self.reset_name = reset_name
        self.reset_edge = reset_edge
        self.reset_bit = reset_bit
        self.senslist = senslist
    def getClockName(self):
        return self.clock_name
    def getClockEdge(self):
        return self.clock_edge
    def getClockBit(self):
        return self.clock_bit
    def getResetName(self):
        return self.reset_name
    def getResetEdge(self):
        return self.reset_edge
    def getResetBit(self):
        return self.reset_bit
    def isClockEdge(self):
        if self.clock_name != '' and self.clock_edge == 'posedge': return True
        if self.clock_name != '' and self.clock_edge == 'negedge': return True
        return False
    def isCombination(self):
        if self.clock_name is None: return True
        return False
    def isResetEdge(self):
        if self.reset_name != '' and self.reset_edge == 'posedge': return True
        if self.reset_name != '' and self.reset_edge == 'negedge': return True
        return False

################################################################################
class Label(object):
    def __init__(self, name):
        self.name = name
        self.cnt = 0
    def get(self):
        ret = self.name + str(self.cnt)
        self.inc()
        return ret
    def inc(self):
        self.cnt += 1

class Labels(object):
    def __init__(self):
        self.labels = {}
    def get(self, name):
        if not name in self.labels: self.labels[name] = Label(name)
        return self.labels[name].get()
    def inc(self, name):
        if not name in self.labels: self.labels[name] = Label(name)
        self.labels[name].inc()

################################################################################
class VariableTable(object):
    def __init__(self):
        self.dict = collections.OrderedDict()
    def add(self, name, var):
        if name in self.dict:
            self.dict[name] = self.dict[name] + (var, )
        else:
            self.dict[name] = (var, )
    def get(self, name):
        return self.dict[name]
    def has(self, name):
        return name in self.dict
    def update(self, table):
        self.dict.update(table)
    def getDict(self):
        return self.dict
        
class SignalTable(VariableTable): pass
class ConstTable(VariableTable): pass
class GenvarTable(VariableTable): pass

class Variables(object):
    def __init__(self):
        self.signal = SignalTable()
        self.const = ConstTable()
        self.genvar = GenvarTable()
    def addSignal(self, name, var):
        if self.const.has(name): return
        self.signal.add(name, var)
    def addConst(self, name, var):
        if self.const.has(name): return
        self.const.add(name, var)
    def addGenvar(self, name, var):
        if self.const.has(name): return
        self.genvar.add(name, var)
    def getSignal(self, name):
        return self.signal.get(name)
    def getConst(self, name):
        return self.const.get(name)
    def getGenvar(self, name):
        return self.genvar.get(name)
    def hasSignal(self, name):
        return self.signal.has(name)
    def hasConst(self, name):
        return self.const.has(name)
    def updateSignal(self, var):
        self.signal.update(var)
    def updateConst(self, var):
        self.const.update(var)
    def getSignals(self):
        return self.signal.getDict()
    def getConsts(self):
        return self.const.getDict()

################################################################################
class DefinitionInfo(object):
    def __init__(self, name, definition):
        self.name = name
        self.definition = definition
        self.variables = Variables()
        self.ioports = []
        self.params = []
    def addSignal(self, name, var):
        self.variables.addSignal(name, var)
    def addConst(self, name, var):
        self.variables.addConst(name, var)
    def addParamName(self, name):
        self.params.append(name)
    def addPort(self, port):
        self.ioports.append(port)
    def addPorts(self, ports):
        for p in ports:
            if isinstance(p, Ioport):
                self.ioports.append(p.first.name)
            else:
                self.ioports.append(p.name)
    def getSignals(self):
        return self.variables.getSignals()
    def getConsts(self):
        return self.variables.getConsts()
    def getDefinition(self):
        return self.definition
    def getIOPorts(self):
        return tuple(self.ioports)
    def getParamNames(self):
        return tuple(self.params)

class DefinitionInfoTable(object):
    def __init__(self):
        self.dict = collections.OrderedDict()
        self.current = None
    def addDefinition(self, name, definition):
        if name in self.dict:
            raise verror.DefinitionError('Already defined: %s' % name)
        self.dict[name] = DefinitionInfo(name, definition)
        self.current = name
    def addPorts(self, ports):
        self.dict[self.current].addPorts(ports)
    def addPort(self, port):
        self.dict[self.current].addPort(port)
    def addSignal(self, name, var):
        self.dict[self.current].addSignal(name, var)
    def addConst(self, name, var):
        self.dict[self.current].addConst(name, var)
    def addParamName(self, name):
        self.dict[self.current].addParamName(name)

    def getSignals(self, name):
        if name not in self.dict: raise verror.DefinitionError('No such module: %s' % name)
        return self.dict[name].getSignals()
    def getConsts(self, name):
        if name not in self.dict: raise verror.DefinitionError('No such module: %s' % name)
        return self.dict[name].getConsts()
    def getDefinition(self, name):
        if name not in self.dict: raise verror.DefinitionError('No such module: %s' % name)
        return self.dict[name].getDefinition()
    def getDefinitions(self):
        return self.dict
    def getIOPorts(self, name):
        if name not in self.dict: raise verror.DefinitionError('No such module: %s' % name)
        return self.dict[name].getIOPorts()
    def getParamNames(self, name):
        if name not in self.dict: raise verror.DefinitionError('No such module: %s' % name)
        return self.dict[name].getParamNames()
    def get_names(self):
        ret = []
        for dk, dv in self.dict.items():
            ret.append(dk)
        return ret

    def overwriteDefinition(self, name, definition):
        self.dict[name] = definition

    def copyDefinition(self, f, t):
        self.dict[t] = copy.deepcopy(self.dict[f])
        self.dict[t].definition.name = t
        self.dict[t].name = t

class ModuleInfo(DefinitionInfo): pass
class ModuleInfoTable(DefinitionInfoTable): pass
class FunctionInfo(DefinitionInfo): pass
class FunctionInfoTable(DefinitionInfoTable): pass
class TaskInfo(DefinitionInfo): pass
class TaskInfoTable(DefinitionInfoTable): pass

################################################################################
frametype_list = ('ifthen', 'ifelse', 'case', 'for', 'while', 'none')

class Frame(object):
    def __init__(self, name, previous, frametype='none',
                 alwaysinfo=None, condition=None,
                 module=False, functioncall=False, taskcall=False,
                 generate=False, always=False, initial=False, loop=None, loop_iter=None,
                 modulename=None):
        self.name = name
        self.previous = previous
        self.next = []
        self.frametype = frametype

        self.alwaysinfo = alwaysinfo
        self.condition = condition

        self.module = module
        self.functioncall = functioncall
        self.taskcall = taskcall
        self.generate = generate
        self.always = always
        self.initial = initial
        self.loop = loop
        self.loop_iter = loop_iter

        self.variables = Variables()
        self.functions = FunctionInfoTable()
        self.tasks = TaskInfoTable()
        self.blockingassign = collections.OrderedDict()
        self.nonblockingassign = collections.OrderedDict()

        self.modulename = modulename

    def getName(self):
        return self.name
    def getPrevious(self):
        return self.previous
    def getNext(self):
        return self.next
    def getFrametype(self):
        return self.frametype

    def isIfelse(self):
        return self.frametype == 'ifelse'

    def getAlwaysInfo(self):
        return self.alwaysinfo
    def getCondition(self):
        return self.condition

    def isModule(self):
        return self.module
    def isFunctioncall(self):
        return self.functioncall
    def isTaskcall(self):
        return self.taskcall
    def isGenerate(self):
        return self.generate
    def isAlways(self):
        return self.always
    def isInitial(self):
        return self.initial

    def setNext(self, nextframe):
        self.next.append(nextframe)

    def setAlwaysInfo(self, clock_name, clock_edge, clock_bit,
                      reset_name, reset_edge, reset_bit, senslist):
        self.alwaysinfo = AlwaysInfo(clock_name, clock_edge, clock_bit,
                                     reset_name, reset_edge, reset_bit, senslist)

    def addSignal(self, node):
        self.variables.addSignal(node.name, node)
    def addConst(self, node):
        self.variables.addConst(node.name, node)
    def addGenvar(self, node):
        self.variables.addGenvar(node.name, 0)

    def addFunction(self, node):
        self.functions.addDefinition(node.name, node)
    def addTask(self, node):
        self.tasks.addDefinition(node.name, node)

    def addFunctionPort(self, node):
        self.functions.addPort(node)
    def addTaskPort(self, node):
        self.tasks.addPort(node)

    def updateSignal(self, signal):
        self.variables.updateSignal(signal)
    def updateConst(self, const):
        self.variables.updateConst(const)

    def getConstant(self, name):
        return self.variables.getConst(name)
    def hasConstant(self, name):
        return self.variables.hasConst(name)

    def getSignal(self, name):
        return self.variables.getSignal(name)
    def hasSignal(self, name):
        return self.variables.hasSignal(name)

    def getSignals(self):
        return self.variables.getSignals()
    def getConsts(self):
        return self.variables.getConsts()
    def getFunctions(self):
        return self.functions.getDefinitions()
    def getTasks(self):
        return self.tasks.getDefinitions()

    def getModuleName(self):
        return self.modulename

    ############################################################################
    def setBlockingAssign(self, dst, bind):
        if not dst in self.blockingassign:
            self.blockingassign[dst] = (bind,)
            return
        current = self.blockingassign[dst]
        for c_i, c in enumerate(current):
            if c.msb == bind.msb and c.msb == bind.msb and c.ptr == bind.ptr:
                self.blockingassign[dst][c_i].tree = bind.tree
                return
        self.blockingassign[dst] = current + (bind,)
            
    def getBlockingAssign(self, dst):
        if dst in self.blockingassign: return self.blockingassign[dst]
        return ()

    def getBlockingAssigns(self):
        return self.blockingassign

    ############################################################################
    def addNonblockingAssign(self, dst, bind):
        if not dst in self.nonblockingassign:
            self.nonblockingassign[dst] = (copy.deepcopy(bind),)
            return
        current = self.nonblockingassign[dst]
        for c_i, c in enumerate(current):
            if c.msb == bind.msb and c.msb == bind.msb and c.ptr == bind.ptr:
                self.nonblockingassign[dst][c_i].tree = copy.deepcopy(bind.tree)
                return
        self.nonblockingassign[dst] = current + (copy.deepcopy(bind),)

    def getNonblockingAssigns(self):
        return self.nonblockingassign

################################################################################
class FrameTable(object):
    def __init__(self):
        self.dict = {}
        self.current = ScopeChain()

        self.function_def = False
        self.task_def = False
        self.for_pre = False
        self.for_post = False
        self.for_iter = None

    def toScopeChain(self, scopename):
        if scopename is None: return self.current
        return self.current + scopename

    def addFrame(self, scopename,
                 frametype='none',
                 alwaysinfo=None, condition=None,
                 module=False, functioncall=False, taskcall=False,
                 generate=False, always=False, initial=False, loop=None, loop_iter=None,
                 modulename=None):

        scopechain = self.toScopeChain(scopename)
        if scopechain in self.dict:
            raise verror.DefinitionError('Already Exists: %s' % str(scopechain))
        ret = self.current
        previous = self.current
        if len(previous) > 0:
            self.dict[previous].setNext(scopechain)
        self.dict[scopechain] = Frame(scopechain, previous, frametype=frametype,
                                      alwaysinfo=alwaysinfo, condition=condition,
                                      module=module, functioncall=functioncall,
                                      taskcall=taskcall, generate=generate,
                                      always=always, initial=initial, loop=loop, loop_iter=loop_iter,
                                      modulename=modulename)
        self.current = scopechain
        return ret

    def hasFrame(self, scope):
        return scope in self.dict

    def getFrametype(self):
        return self.dict[self.current].getFrametype()
    def getAlwaysInfo(self):
        return self.dict[self.current].getAlwaysInfo()
    def getCondition(self):
        return self.dict[self.current].getCondition()

    def isModule(self):
        return self.dict[self.current].isModule()
    def isFunctioncall(self):
        return self.dict[self.current].isFunctioncall()
    def isTaskcall(self):
        return self.dict[self.current].isTaskcall()
    def isGenerate(self):
        return self.dict[self.current].isGenerate()
    def isAlways(self):
        return self.dict[self.current].isAlways()
    def isInitial(self):
        return self.dict[self.current].isInitial()
    def getLoop(self):
        return self.dict[self.current].loop
    def getLoopIter(self):
        return self.dict[self.current].loop_iter

    def isFunctiondef(self):
        return self.function_def
    def isTaskdef(self):
        return self.task_def
    def isForpre(self):
        return self.for_pre
    def isForpost(self):
        return self.for_post

    def setFunctionDef(self):
        self.function_def = True
    def unsetFunctionDef(self):
        self.function_def = False
    def setTaskDef(self):
        self.task_def = True
    def unsetTaskDef(self):
        self.task_def = False
    def setForPre(self):
        self.for_pre = True
    def unsetForPre(self):
        self.for_pre = False
    def setForPost(self):
        self.for_post = True
    def unsetForPost(self):
        self.for_post = False

    def setForIter(self, iter):
        self.for_iter = iter
    def getForIter(self):
        return self.for_iter

    def setAlwaysInfo(self, clock_name, clock_edge, clock_bit,
                      reset_name, reset_edge, reset_bit, senslist):
        self.dict[self.current].setAlwaysInfo(clock_name, clock_edge, clock_bit,
                                              reset_name, reset_edge, reset_bit, senslist)

    def setCurrent(self, current):
        self.current = current

    def getCurrent(self):
        return self.current

    def getModuleName(self):
        return self.dict[self.current].getModuleName()

    def getLabelKey(self, name):
        ret = ''
        if self.isModule(): ret += 'md_'
        if self.isGenerate(): ret += 'ge_'
        if self.isAlways(): ret += 'al_'
        if self.isInitial(): ret += 'in_'
        if self.isFunctioncall(): ret += 'fc_'
        if self.isTaskcall(): ret += 'tc_'
        return ret + name

    ############################################################################
    def addSignal(self, var):
        if self.function_def and isinstance(var, Input):
            self.dict[self.current].addFunctionPort(var)
            return
        if self.task_def and isinstance(var, Input):
            self.dict[self.current].addTaskPort(var)
            return
        self.dict[self.current].addSignal(var)
            
    def addConst(self, var):
        self.dict[self.current].addConst(var)
    def addGenvar(self, var):
        self.dict[self.current].addGenvar(var)

    def addFunction(self, var):
        self.dict[self.current].addFunction(var)
    def addTask(self, var):
        self.dict[self.current].addTask(var)

    ############################################################################
    def updateSignal(self, signal):
        self.dict[self.current].updateSignal(signal)
    def updateConst(self, const):
        self.dict[self.current].updateConst(const)

    ############################################################################
    def getAllInstances(self):
        ret = []
        for dk in self.dict.keys():
            if dk[-1].scopetype == 'module':
                ret.append( (dk, self.dict[dk].getModuleName()) )
        return tuple(ret)
        
    def getAllSignals(self):
        ret = collections.OrderedDict()
        for dk, dv in self.dict.items():
            ret.update(
                map_key((lambda x: dk+ScopeLabel(x,'signal')), dv.getSignals()))
        return ret

    def getAllConsts(self):
        ret = collections.OrderedDict()
        for dk, dv in self.dict.items():
            ret.update(
                map_key((lambda x: dk+ScopeLabel(x,'signal')), dv.getConsts()))
        return ret

    def getAllFunctions(self):
        ret = collections.OrderedDict()
        for dk, dv in self.dict.items():
            ret.update(
                map_key((lambda x: dk+ScopeLabel(x,'function')), dv.getFunctions()))
        return ret

    def getAllTasks(self):
        ret = collections.OrderedDict()
        for dk, dv in self.dict.items():
            ret.update(
                map_key((lambda x: dk+ScopeLabel(x,'task')), dv.getTasks()))
        return ret

    ############################################################################
    def getSignals(self, scope):
        return map_key(
            (lambda x: scope+ScopeLabel(x,'signal')), self.dict[scope].getSignals())

    def getConsts(self, scope):
        return map_key(
            (lambda x: scope+ScopeLabel(x,'signal')), self.dict[scope].getConsts())

    def getFunctions(self, scope):
        return map_key(
            (lambda x: scope+ScopeLabel(x,'function')), self.dict[scope].getFunctions())

    def getTasks(self, scope):
        return map_key(
            (lambda x: scope+ScopeLabel(x,'task')), self.dict[scope].getTasks())

    ################################################################################
    def getGenerateConditions(self):
        ret = []
        ptr = self.current
        while True:
            frame = self.dict[ptr]
            if not frame.isGenerate(): break
            if frame.loop is not None: # generate for
                ret.append( (frame.loop_iter, frame.loop) )
            elif frame.condition is not None: # generate if
                if frame.isIfelse():
                    ret.append( (None, Ulnot(frame.condition)) )
                else:
                    ret.append( (None, frame.condition) )
            ptr = self.dict[ptr].previous
        return tuple(reversed(ret))

    def getAlwaysStatus(self):
        ptr = self.current
        frame = self.dict[ptr]
        alwaysinfo = frame.getAlwaysInfo()
        while True:
            if frame.isFunctioncall(): return None
            if frame.isTaskcall(): return None
            if not frame.isAlways(): return None
            if alwaysinfo is not None: return alwaysinfo
            ptr = self.dict[ptr].previous
            frame = self.dict[ptr]
            alwaysinfo = frame.getAlwaysInfo()
    
    def setBlockingAssign(self, dst, bind, scope):
        self.dict[scope].setBlockingAssign(dst, bind)

    def getBlockingAssign(self, dst, scope):
        p = scope
        while self.dict[p].isAlways():
            ret = self.dict[p].getBlockingAssign(dst)
            if ret: return ret
            p = self.dict[p].getPrevious()
        return ()

    def addNonblockingAssign(self, dst, bind):
        self.dict[self.current].addNonblockingAssign(dst, bind)

    def getBlockingAssigns(self):
        return self.dict[self.current].getBlockingAssigns()

    def getBlockingAssignsScope(self, scope):
        return self.dict[scope].getBlockingAssigns()

    def getNonblockingAssigns(self):
        return self.dict[self.current].getNonblockingAssigns()

    def getPreviousNonblockingAssign(self):
        previous = self.dict[self.current].previous
        if len(previous) == 0: return collections.OrderedDict()
        previous_frame = self.dict[previous]
        if not previous_frame.isAlways(): return collections.OrderedDict()
        return previous_frame.getNonblockingAssigns()

    def searchConstantDefinition(self, key, name):
        if self.dict[key].hasConstant(name):
            return key, self.dict[key].getConstant(name)
        if self.dict[key].isModule(): return None, None
        previous = self.dict[key].getPrevious()
        return self.searchConstantDefinition(previous, name)

    def getConstantDefinition(self, key, name):
        if self.dict[key].hasConstant(name):
            return self.dict[key].getConstant(name)
        return None

    def searchSignalDefinition(self, key, name):
        if self.dict[key].hasSignal(name):
            return key, self.dict[key].getSignal(name)
        if self.dict[key].isModule(): return None, None
        previous = self.dict[key].getPrevious()
        return self.searchSignalDefinition(previous, name)

    def searchMatchedScopeChain(self, currentchain, targetchain):
        skiplength = len(currentchain) - 1
        printable = currentchain[skiplength].isPrintable()

        if printable:
            if currentchain[skiplength] != targetchain[0]: # not match
                return None
            if len(targetchain) == 1:
                return ScopeChain( [targetchain[0]] )
            for nextframe in self.dict[currentchain].getNext():
                rslt = self.searchMatchedScopeChain(nextframe, targetchain[1:])
                if rslt is not None:
                    return ScopeChain( [currentchain[skiplength]] ) + rslt
            return None

        if (currentchain[skiplength].scopetype == 'for' and 
            targetchain[0].scopetype == 'for'):
            if currentchain[skiplength].scopeloop != targetchain[0].scopeloop:
                return None
            if len(targetchain) == 1:
                return ScopeChain( [targetchain[0]] )
            for nextframe in self.dict[currentchain].getNext():
                rslt = self.searchMatchedScopeChain(nextframe, targetchain[1:])
                if rslt is not None:
                    return ScopeChain( [currentchain[skiplength]] ) + rslt
                    
            return None

        for nextframe in self.dict[currentchain].getNext():
            rslt = self.searchMatchedScopeChain(nextframe, targetchain)
            if rslt is not None:
                return ScopeChain( [currentchain[skiplength]] ) + rslt
        return None




#-------------------------------------------------------------------------------
# merge.py
# 
# Merging multiple dataflow definitions into a single dataflow definition
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

import pyverilog.utils.version
import pyverilog.utils.verror as verror
import pyverilog.utils.signaltype as signaltype

if sys.version_info[0] >= 3:
    from pyverilog.dataflow.dataflow import *
    from pyverilog.dataflow.visit import *
    from pyverilog.dataflow.optimizer import VerilogOptimizer
    import pyverilog.dataflow.reorder as reorder
else:
    from dataflow import *
    from visit import *
    from optimizer import VerilogOptimizer
    import reorder

class VerilogDataflowMerge(object):
    def __init__(self, topmodule, terms, binddict, resolved_terms, resolved_binddict, constlist):
        self.topmodule = topmodule
        self.terms = terms
        self.binddict = binddict
        self.resolved_terms = resolved_terms
        self.resolved_binddict = resolved_binddict
        self.constlist = constlist
        self.optimizer = VerilogOptimizer(terms, constlist)

    ############################################################################
    def getTerm(self, termname):
        if not termname in self.terms: return None
        return self.terms[termname]

    def getBindlist(self, termname):
        if not termname in self.binddict: return ()
        return self.binddict[termname]

    def getResolvedTerm(self, termname):
        if not termname in self.resolved_terms: return None
        return self.resolved_terms[termname]

    def getResolvedBindlist(self, termname):
        if not termname in self.resolved_binddict: return ()
        return self.resolved_binddict[termname]

    ############################################################################
    def getTermtype(self, termname):
        term = self.getTerm(termname)
        if term is None: raise verror.DefinitionError('No such Term: %s' % termname)
        return term.termtype

    ############################################################################
    def getAssignType(self, termname, bind):
        termtype = self.getTermtype(termname)
        if signaltype.isWire(termtype):
            return 'assign'
        if signaltype.isWireArray(termtype):
            return 'assign'
        if signaltype.isReg(termtype):
            if bind.isClockEdge(): return 'clockedge'
            return 'combination'
        if signaltype.isRegArray(termtype):
            if bind.isClockEdge(): return 'clockedge'
            return 'combination'
        if signaltype.isInteger(termtype):
            if bind.isClockEdge(): return 'clockedge'
            return 'combination'
        if signaltype.isParameter(termtype):
            return 'parameter'
        if signaltype.isLocalparam(termtype):
            return 'localparam'
        if signaltype.isOutput(termtype):
            return 'assign'
        if signaltype.isInput(termtype):
            return 'assign'
        if signaltype.isFunction(termtype):
            return 'assign'
        if signaltype.isRename(termtype):
            return 'assign'
        if signaltype.isGenvar(termtype):
            return 'genvar'
        raise verror.DefinitionError('Unexpected Assignment Type: %s : %s' % (str(termname), str(termtype)))

    ############################################################################
    def isCombination(self,termname):
        bindlist = self.getBindlist(termname)
        if bindlist is None: return False
        for bind in bindlist:
            if bind.isCombination(): return True
        return False

    ############################################################################
    def getTree(self, termname, ptr=None):
        bindlist = self.getResolvedBindlist(termname)
        bindlist = self.getOptimizedBindlist(bindlist)
        if bindlist is None: return None
        if len(bindlist) == 0: return None

        termtype = self.getTermtype(termname)

        if signaltype.isRegArray(termtype) or signaltype.isWireArray(termtype):
            discretebinds = {}
            for bind in bindlist:
                if isinstance(bind.ptr, DFEvalValue):
                    ptrval = bind.ptr.value
                    if not ptrval in discretebinds: discretebinds[ptrval] = []
                    discretebinds[ptrval] += [bind]
                else:
                    if not 'any' in discretebinds: discretebinds['any'] = []
                    discretebinds['any'] += [bind]

            if 'any' in discretebinds:
                return DFTerminal(termname)

            if isinstance(ptr, DFEvalValue):
                if len(discretebinds[ptr.value]) == 0:
                    return None
                if len(discretebinds[ptr.value]) == 1:
                    return discretebinds[ptr.value][0].tree
                return self.getMergedTree(discretebinds[ptr.value])

            minptr = min(list(discretebinds.keys()))
            maxptr = max(list(discretebinds.keys()))
            ret = None
            for c in range(minptr, maxptr+1):
                truetree = None
                if len(discretebinds[c]) == 0:
                    continue
                if len(discretebinds[c]) == 1:
                    truetree = discretebinds[c][0].tree
                else:
                    truetree = self.getMergedTree(discretebinds[c])
                ret = DFBranch(DFOperator((DFEvalValue(c), ptr),'Eq'), truetree, ret)
            return ret

        if len(bindlist) == 1:
            return bindlist[0].tree
        new_tree = self.getMergedTree(bindlist)
        return self.optimizer.optimize(new_tree) 

    def getResolvedTree(self, termname, ptr=None):
        raise verror.ImplementationError()

    ############################################################################        
    def isClockEdge(self, termname, msb=None, lsb=None, ptr=None):
        bind = self.binddict[termname]
        return bind[0].isClockEdge()

    ############################################################################
    def getSources(self, tree):
        if tree is None: return set()
        if isinstance(tree, DFConstant): return set()
        if isinstance(tree, DFUndefined): return set()
        if isinstance(tree, DFEvalValue): return set()
        if isinstance(tree, DFTerminal):
            return set( [tree.name,] )
        if isinstance(tree, DFBranch):
            ret = set()
            ret |= self.getSources(tree.condnode)
            ret |= self.getSources(tree.truenode)
            ret |= self.getSources(tree.falsenode)
            return ret
        if isinstance(tree, DFOperator):
            nextnodes = []
            for n in tree.nextnodes:
                nextnodes.extend(self.getSources(n))
            return set(nextnodes)
        if isinstance(tree, DFPartselect):
            ret = set()
            ret |= self.getSources(tree.var)
            ret |= self.getSources(tree.msb)
            ret |= self.getSources(tree.lsb)
            return ret
        if isinstance(tree, DFPointer):
            ret = set()
            ret |= self.getSources(tree.var)
            ret |= self.getSources(tree.ptr)
            return ret
        if isinstance(tree, DFConcat):
            nextnodes = []
            for n in tree.nextnodes:
                nextnodes.extend(self.getSources(n))
            return set(nextnodes)
        if isinstance(tree, DFDelay):
            ret = set()
            ret |= self.getSources(tree.nextnode)
            return ret
        raise verror.DefinitionError('Undefined Node Type: %s : %s' % (str(type(tree)), str(tree)))

    ################################################################################
    def getBindSources(self, termname):
        sources = set()
        sources |= self.getTermSources(termname)
        sources |= self.getBindinfoSources(termname)
        return sources

    ################################################################################
    def getTermSources(self, termname):
        term = self.getTerm(termname)
        if term is None: return set()
        sources = set()
        sources |= self.getTreeSources(term.msb)
        sources |= self.getTreeSources(term.lsb)
        sources |= self.getTreeSources(term.lenmsb)
        sources |= self.getTreeSources(term.lenlsb)
        return sources
 
    def getBindinfoSources(self, termname):
        bindlist = self.getBindlist(termname)
        sources = set()
        for bind in bindlist:
            sources |= self.getTreeSources(bind.msb)
            sources |= self.getTreeSources(bind.lsb)
            sources |= self.getTreeSources(bind.ptr)
            sources |= self.getTreeSources(bind.tree)
        return sources

    def getTreeSources(self, tree):
        if tree is None: return set()
        if isinstance(tree, DFConstant): return set()
        if isinstance(tree, DFUndefined): return set()
        if isinstance(tree, DFEvalValue): return set()
        if isinstance(tree, DFTerminal):
            return set( [tree.name,] )
        if isinstance(tree, DFBranch):
            ret = set()
            ret |= self.getTreeSources(tree.condnode)
            ret |= self.getTreeSources(tree.truenode)
            ret |= self.getTreeSources(tree.falsenode)
            return ret
        if isinstance(tree, DFOperator):
            ret = set()
            for n in tree.nextnodes:
                ret |= self.getTreeSources(n)
            return ret
        if isinstance(tree, DFPartselect):
            ret = set()
            ret |= self.getTreeSources(tree.var)
            ret |= self.getTreeSources(tree.msb)
            ret |= self.getTreeSources(tree.lsb)
            return ret
        if isinstance(tree, DFPointer):
            ret = set()
            ret |= self.getTreeSources(tree.var)
            ret |= self.getTreeSources(tree.ptr)
            return ret
        if isinstance(tree, DFConcat):
            ret = set()
            for n in tree.nextnodes:
                ret |= self.getTreeSources(n)
            return ret
        raise verror.DefinitionError('Undefined Node Type: %s : %s' % (str(type(tree)), str(tree)))

    ################################################################################
    def getMergedTree(self, optimized_bindlist):
        concatlist = []
        last_msb = -1
        last_ptr = -1
        def bindkey(x):
            lsb = 0 if x.lsb is None else x.lsb.value
            ptr = 0 if not isinstance(x.ptr, DFEvalValue) else x.ptr.value
            term = self.getTerm(x.dest)
            length = abs(self.optimizer.optimize(term.msb).value - self.optimizer.optimize(term.lsb).value) + 1
            return ptr * length + lsb
        for bind in sorted(optimized_bindlist, key=bindkey):
            lsb = 0 if bind.lsb is None else bind.lsb.value
            if last_ptr != (-1 if not isinstance(bind.ptr, DFEvalValue) else bind.ptr.value):
                continue
            if last_msb + 1 < lsb:
                concatlist.append(DFUndefined(last_msb-lsb-1))
            concatlist.append(bind.tree)
            last_msb = -1 if bind.msb is None else bind.msb.value
            last_ptr = -1 if not isinstance(bind.ptr, DFEvalValue) else bind.ptr.value
        return DFConcat(tuple(reversed(concatlist)))

    ################################################################################
    def getOptimizedBindlist(self, bindlist):
        if len(bindlist) == 0: return ()
        new_bindlist = []
        for bind in bindlist:
            tree = self.optimizer.optimize(bind.tree)
            msb = self.optimizer.optimize(bind.msb)
            lsb = self.optimizer.optimize(bind.lsb)
            ptr = self.optimizer.optimize(bind.ptr)
            new_bind = copy.deepcopy(bind)
            new_bind.tree = tree
            new_bind.msb = msb
            new_bind.lsb = lsb
            new_bind.ptr = ptr
            new_bindlist.append(new_bind)
        if len(new_bindlist) == 1: return (new_bindlist[0],)
        split_positions = self.splitPositions(tuple(new_bindlist))
        new_bindlist = self.splitBindlist(tuple(new_bindlist), split_positions)
        return self.mergeBindlist(tuple(new_bindlist))

    def mergeBindlist(self, bindlist):
        merged_bindlist = []
        last_bind = None

        def bindkey(x):
            lsb = 0 if x.lsb is None else x.lsb.value
            ptr = 0 if not isinstance(x.ptr, DFEvalValue) else x.ptr.value
            term = self.getTerm(x.dest)
            length = abs(self.optimizer.optimize(term.msb).value - self.optimizer.optimize(term.lsb).value) + 1
            return ptr * length + lsb

        for bind in sorted(bindlist, key=bindkey):
            if last_bind is None:
                merged_bindlist.append(copy.deepcopy(bind))
                last_bind = copy.deepcopy(bind)
            elif isinstance(last_bind.ptr, DFEvalValue) and isinstance(bind.ptr, DFEvalValue) and last_bind.ptr.value != bind.ptr.value:
                merged_bindlist.append(copy.deepcopy(bind))
                last_bind = copy.deepcopy(bind)
            elif last_bind.lsb is None or bind.lsb is None or last_bind is None or bind.msb is None:
                merged_bindlist.append(copy.deepcopy(bind))
                last_bind = copy.deepcopy(bind)
            elif last_bind.lsb.value == bind.lsb.value and last_bind.msb.value == bind.msb.value:
                new_tree = self.mergeTree(last_bind.tree, bind.tree)
                new_tree = self.optimizer.optimize(new_tree)
                merged_bindlist.pop()
                new_bind = copy.deepcopy(bind)
                new_bind.tree = new_tree
                merged_bindlist.append(new_bind)
                last_bind = copy.deepcopy(new_bind)
            else:
                merged_bindlist.append(copy.deepcopy(bind))
                last_bind = copy.deepcopy(bind)
        return tuple(merged_bindlist)

    def mergeTree(self, first, second):
        if isinstance(first, DFBranch) and isinstance(second, DFBranch):
            cond_fst = self.optimizer.optimize(first.condnode)
            cond_snd = self.optimizer.optimize(second.condnode)
            if cond_fst == cond_snd:
                return DFBranch(cond_fst, self.mergeTree(first.truenode, second.truenode), self.mergeTree(first.falsenode, second.falsenode))
            appended = copy.deepcopy(first)
            return DFBranch(cond_snd, self.appendTail(appended, second.truenode), self.appendTail(appended, second.falsenode))

        if first is not None and second is None:
            return first
        if first is None and second is not None:
            return second

        if isinstance(first, DFBranch) and second is None:
            return first
        if first is None and isinstance(second, DFBranch):
            return second

        if isinstance(first, DFBranch) and not isinstance(second, DFBranch):
            cond_fst = self.optimizer.optimize(first.condnode)
            appended = copy.deepcopy(second)
            return DFBranch(cond_fst, self.appendTail(appended, first.truenode), self.appendTail(appended, first.falsenode))
        if not isinstance(first, DFBranch) and isinstance(second, DFBranch):
            cond_snd = self.optimizer.optimize(second.condnode)
            appended = copy.deepcopy(first)
            return DFBranch(cond_snd, self.appendTail(appended, second.truenode), self.appendTail(appended, second.falsenode))

        if not isinstance(first, DFBranch) and not isinstance(second, DFBranch):
            return second

        raise verror.FormatError('Can not merge trees.')

    def appendTail(self, appended, target):
        if target is None:
            return copy.deepcopy(appended)
        if isinstance(target, DFBranch):
            return DFBranch(target.condnode, self.appendTail(appended, target.truenode), self.appendTail(appended, target.falsenode))
        return target

    def splitBindlist(self, bindlist, split_positions):
        if len(bindlist) == 0: return ()
        return self.splitBindPositions(bindlist[0], split_positions) + self.splitBindlist(bindlist[1:], split_positions)

    def splitBindPositions(self, bind, split_positions):
        if len(split_positions) == 0: return (copy.deepcopy(bind),)
        if bind is None: return (copy.deepcopy(bind),)
        bind_left, bind_right = self.splitBind(bind, split_positions[0])
        ret = () if bind_right is None else (bind_right,)
        return ret + self.splitBindPositions(bind_left, split_positions[1:])

    def splitBind(self, bind, splitpos):
        tree = bind.tree
        msb = self.optimizer.optimizeConstant(bind.msb)
        lsb = self.optimizer.optimizeConstant(bind.lsb)
        ptr = self.optimizer.optimizeConstant(bind.ptr)
        if ptr is not None and msb is None or lsb is None:
            termtype = self.getTermtype(bind.dest)
            if signaltype.isRegArray(termtype) or signaltype.isWireArray(termtype):
                msb = self.optimizer.optimizeConstant(copy.deepcopy(term.msb))
                lsb = self.optimizer.optimizeConstant(copy.deepcopy(term.lsb))
            else:
                msb = copy.deepcopy(ptr)
                lsb = copy.deepcopy(ptr)
        if ptr is None and msb is None or lsb is None:
            term = self.getTerm(bind.dest)
            msb = self.optimizer.optimizeConstant(copy.deepcopy(term.msb))
            lsb = self.optimizer.optimizeConstant(copy.deepcopy(term.lsb))
        if splitpos > lsb.value and splitpos <= msb.value: # split
            right_lsb = lsb.value
            right_msb = splitpos - 1
            right_width = splitpos - lsb.value
            left_lsb = splitpos
            left_msb = msb.value
            left_width = msb.value - splitpos + 1
            right_tree = reorder.reorder(DFPartselect(copy.deepcopy(tree), DFEvalValue(right_width-1), DFEvalValue(0)))
            left_tree = reorder.reorder(DFPartselect(copy.deepcopy(tree), DFEvalValue(msb.value), DFEvalValue(msb.value-left_width+1)))
            right_tree = self.optimizer.optimize(right_tree)
            left_tree = self.optimizer.optimize(left_tree)
            left_bind = copy.deepcopy(bind)
            left_bind.tree = left_tree
            left_bind.msb = DFEvalValue(left_msb)
            left_bind.lsb = DFEvalValue(left_lsb)
            right_bind = copy.deepcopy(bind)
            right_bind.tree = right_tree
            right_bind.msb = DFEvalValue(right_msb)
            right_bind.lsb = DFEvalValue(right_lsb)
            return left_bind, right_bind
        return bind, None

    def splitPositions(self, bindlist):
        split_positions = set([])
        assigned_range = [] # (msb, lsb, ptr)

        for bind in bindlist:
            ptr = self.optimizer.optimizeConstant(bind.ptr)
            msb = self.optimizer.optimizeConstant(bind.msb)
            lsb = self.optimizer.optimizeConstant(bind.lsb)
            if msb is None and lsb is None: 
                term = self.getTerm(bind.dest)
                msb = self.optimizer.optimizeConstant(term.msb)
                lsb = self.optimizer.optimizeConstant(term.lsb)
            elif not isinstance(msb, DFEvalValue) or not isinstance(lsb, DFEvalValue):
                raise FormatError('MSB and LSB should be constant.')
            
            if ptr is None or isinstance(ptr, DFEvalValue):
                ptrval = None if ptr is None else ptr.value
                matched_range = self.matchedRange(tuple(assigned_range), msb.value, lsb.value, ptrval)
                unmatched_range = self.unmatchedRange(tuple(matched_range), msb.value, lsb.value, ptrval)
                split_positions |= self.getPositionsFromRange(matched_range, ptrval)
                assigned_range += matched_range + unmatched_range

        return tuple(sorted(list(split_positions)))
    
    def getPositionsFromRange(self, matched_range, search_ptr):
        positions = set([])
        for msb, lsb, ptr in matched_range:
            if search_ptr is not None and search != ptr: continue
            positions.add(lsb)
            positions.add(msb+1)
        return positions

    def matchedRange(self, assigned_range, search_msb, search_lsb, search_ptr):
        matched_range = []
        for msb, lsb, ptr in assigned_range:
            match = False
            if search_ptr is not None and ptr != search_ptr: continue
            if lsb <= search_lsb and search_lsb <= msb:
                match = True
                match_lsb = search_lsb
            else:
                match_lsb = lsb
            if lsb <= search_msb and search_msb <= msb:
                match = True
                match_msb = search_msb
            else:
                match_msb = msb
            if match:
                matched_range.append( (match_msb, match_lsb, search_ptr) )
        return tuple(matched_range)

    def unmatchedRange(self, matched_range, search_msb, search_lsb, search_ptr):
        unmatched_range = []
        minval = None
        maxval = None
        last_msb = None
        for msb, lsb, ptr in sorted(matched_range, key=lambda x:x[0]):
            if search_ptr is not None and ptr != search_ptr: continue
            if minval is None or lsb < minval: minval = lsb
            if maxval is None or msb > maxval: maxval = msb
            if last_msb is not None and last_msb + 1 > lsb:
                unmatched_range.append( (last_msb + 1, lsb - 1, ptr) )
            last_msb = msb
        if minval is None and maxval is None: return ( (search_msb, search_lsb, search_ptr), )
        if search_lsb < minval: unmatched_range.append( (search_lsb, minval - 1, search_ptr) )
        if maxval < search_msb: unmatched_range.append( (maxval + 1, search_msb, search_ptr) )
        return tuple(unmatched_range)

################################################################################
if __name__ == '__main__':
    from optparse import OptionParser
    import pyverilog.utils.util as util
    if sys.version_info[0] >= 3:
        from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
        from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
    else:
        from dataflow_analyzer import VerilogDataflowAnalyzer
        from optimizer import VerilogDataflowOptimizer
    INFO = "Dataflow merge module"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python merge.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-I","--include",dest="include",action="append",
                         default=[],help="Include path")
    optparser.add_option("-D",dest="define",action="append",
                         default=[],help="Macro Definition")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("--nobind",action="store_true",dest="nobind",
                         default=False,help="No binding traversal, Default=False")
    optparser.add_option("--noreorder",action="store_true",dest="noreorder",
                         default=False,help="No reordering of binding dataflow, Default=False")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule,
                                       noreorder=options.noreorder,
                                       nobind=options.nobind,
                                       preprocess_include=options.include,
                                       preprocess_define=options.define)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)
    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()

    merge = VerilogDataflowMerge(options.topmodule, terms, binddict,
                                 resolved_terms, resolved_binddict, constlist)

    for target in options.searchtarget:
        termname = util.toTermname(target)
        tree = merge.getTree(termname)
        print('target: %s' % target)
        print(tree.tostr())



#-------------------------------------------------------------------------------
# identifiervisitor.py
# 
# Identifier list generator in a nested operator 
#
# Copyright (C) 2015, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

if sys.version_info[0] >= 3:
    from pyverilog.dataflow.visit import NodeVisitor
else:
    from visit import NodeVisitor

def getIdentifiers(node):
    v = IdentifierVisitor()
    v.visit(node)
    ids = v.getIdentifiers()
    return ids
    
class IdentifierVisitor(NodeVisitor):
    def __init__(self):
        self.identifiers = []

    def getIdentifiers(self):
        return tuple(self.identifiers)

    def reset(self):
        self.identifiers = []
    
    def visit_Identifier(self, node):
        self.identifiers.append(node.name)

if __name__ == '__main__':
    import pyverilog.vparser.ast as vast

    a = vast.Identifier('a')
    b = vast.Identifier('b')
    c = vast.Plus(a, b)

    ids = getIdentifiers(c)
    print(ids)



#-------------------------------------------------------------------------------
# modulevisitor.py
# 
# Module definition visitor
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

import pyverilog.vparser.parser
from pyverilog.vparser.ast import *

if sys.version_info[0] >= 3:
    from pyverilog.dataflow.visit import *
else:
    from visit import *

class ModuleVisitor(NodeVisitor):
    def __init__(self):
        self.moduleinfotable = ModuleInfoTable()

    def visit_ModuleDef(self, node):
        self.moduleinfotable.addDefinition(node.name, node)
        self.generic_visit(node)

    def visit_Portlist(self, node):
        self.moduleinfotable.addPorts(node.ports)

    def visit_Input(self, node):
        self.moduleinfotable.addSignal(node.name, node)

    def visit_Output(self, node):
        self.moduleinfotable.addSignal(node.name, node)

    def visit_Inout(self, node):
        self.moduleinfotable.addSignal(node.name, node)

    def visit_Parameter(self, node):
        self.moduleinfotable.addConst(node.name, node)
        self.moduleinfotable.addParamName(node.name)

    def visit_Locaparam(self, node):
        self.moduleinfotable.addConst(node.name, node)

    # Skip Rule
    def visit_Function(self, node):
        pass

    def visit_Task(self, node):
        pass

    def visit_Always(self, node):
        pass

    def visit_Initial(self, node):
        pass

    def visit_InstanceList(self, node):
        pass

    def visit_Instance(self, node):
        pass

    def visit_Pragma(self, node):
        pass

    # get functions
    def get_modulenames(self):
        return self.moduleinfotable.get_names()

    def get_moduleinfotable(self):
        return self.moduleinfotable



#-------------------------------------------------------------------------------
# scope.py
# 
# classes for definition of scope
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os
import copy

scopetype_list_unprint = ('generate', 'always', 'function', #'functioncall', 
                          'task', 'taskcall', 'initial', 'for', 'while', 'if')
scopetype_list_print = ('module', 'block', 'signal', 'functioncall',)
scopetype_list = scopetype_list_unprint + scopetype_list_print + ('any', )

class ScopeLabel(object):
    def __init__(self, scopename, scopetype='any', scopeloop=None):
        self.scopename = scopename
        if scopetype not in scopetype_list:
            raise DefinitionError('No such Scope type')
        self.scopetype = scopetype
        self.scopeloop = scopeloop
    def __repr__(self):
        ret = []
        ret.append(self.scopename)
        if self.scopeloop is not None:
            ret.append('[')
            ret.append(str(self.scopeloop))
            ret.append(']')
        return ''.join(ret)
    def tocode(self):
        if self.scopetype in scopetype_list_unprint: return ''
        return self.scopename
    def __eq__(self, other):
        if type(self) != type(other): return False
        if self.scopetype == 'any' or other.scopetype == 'any':
            return ((self.scopename, self.scopeloop) ==
                    (other.scopename, other.scopeloop))
        return (self.scopename, self.scopetype, self.scopeloop) == (other.scopename, other.scopetype, other.scopeloop)
    def __ne__(self, other):
        return not self.__eq__(other)
    def __hash__(self):
        #return hash((self.scopename, self.scopetype, self.scopeloop))
        return hash((self.scopename, self.scopeloop)) # to use for dict key with any scopetype
    def isPrintable(self):
        return self.scopetype in (scopetype_list_print + ('any',))

class ScopeChain(object):
    def __init__(self, scopechain=None):
        self.scopechain = []
        if scopechain is not None:
            self.scopechain = scopechain
    def __add__(self, r):
        new_chain = copy.deepcopy(self)
        if isinstance(r, ScopeLabel):
            new_chain.append(r)
        elif isinstance(r, ScopeChain):
            new_chain.extend(r.scopechain)
        else:
            raise verror.DefinitionError('Can not add %s' % str(r))
        return new_chain
    def append(self, r):
        self.scopechain.append(r)
    def extend(self, r):
        self.scopechain.extend(r)
    def tocode(self):
        ret = []
        it = None
        for scope in self.scopechain:
            l = scope.tocode()
            if l:
                ret.append(l)
            if it is not None:
                ret.append(it)
            if l:
                #ret.append('.')
                #ret.append('_dot_')
                ret.append('_')
            if scope.scopetype == 'for' and scope.scopeloop is not None:
                #it = '[' + str(scope.scopeloop) + ']'
                #it = '_L_' + str(scope.scopeloop) + '_R_'
                it = '_' + str(scope.scopeloop) + '_'
            else:
                it = None
        ret = ret[:-1]
        return ''.join(ret)
    def __repr__(self):
        ret = ''
        for scope in self.scopechain:
            l = scope.__repr__()
            ret += l + '.'
        ret = ret[:-1]
        return ret
    def __len__(self):
        return len(self.scopechain)
    def __eq__(self, other):
        if type(self) != type(other): return False
        return self.scopechain == other.scopechain
    def __ne__(self, other):
        return not self.__eq__(other)
    def __hash__(self):
        return hash(tuple(self.scopechain))
    def __getitem__(self, key):
        if isinstance(key, slice):
            indices = key.indices(len(self))
            return ScopeChain([self.scopechain[x] for x in range(*indices)])
        return self.scopechain[key]
    def __iter__(self):
        for scope in self.scopechain:
            yield scope



import sys
if sys.version_info[0] < 3:
    import modulevisitor
    import signalvisitor
    import bindvisitor
    import dataflow_analyzer
    import merge
    import walker
    import subset
    import codegen
    import graphgen



#-------------------------------------------------------------------------------
# reorder.py
# 
# Reorders DFNode trees in order of DFBranch, DFOperator, and DFTerminal
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os
import copy

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

if sys.version_info[0] >= 3:
    from pyverilog.dataflow.dataflow import *
else:
    from dataflow import *

def reorder(tree):
    if tree is None: return None
    if isinstance(tree, DFConstant): return tree
    if isinstance(tree, DFTerminal): return tree
    if isinstance(tree, DFEvalValue): return tree
    if isinstance(tree, DFUndefined): return tree
    if isinstance(tree, DFHighImpedance): return tree

    if isinstance(tree, DFBranch):
        truenode = reorder(tree.truenode)
        falsenode = reorder(tree.falsenode)
        condnode = reorder(tree.condnode)
        if isinstance(condnode, DFBranch):
            return insertBranch(condnode, truenode, falsenode)
        return DFBranch(condnode, truenode, falsenode)

    if isinstance(tree, DFOperator):
        resolvednodes = []
        for n in tree.nextnodes:
            resolvednodes.append(reorder(n))
        for r in resolvednodes:
            if isinstance(r, DFBranch):
                return insertOpList(resolvednodes, tree.operator)
        return DFOperator(tuple(resolvednodes), tree.operator)

    if isinstance(tree, DFConcat):
        resolvednodes = []
        for n in tree.nextnodes:
            resolvednodes.append(reorder(n))
        for r in resolvednodes:
            if isinstance(r, DFBranch):
                return insertConcat(resolvednodes)
        return DFConcat(tuple(resolvednodes))

    if isinstance(tree, DFPartselect):
        resolved_msb = reorder(tree.msb)
        resolved_lsb = reorder(tree.lsb)
        resolved_var = reorder(tree.var)
        if isinstance(resolved_msb, DFBranch) or isinstance(resolved_lsb, DFBranch):
            raise FormatError('MSB and LSB should not be DFBranch')
        if isinstance(resolved_var, DFBranch):
            return insertPartselect(resolved_var, resolved_msb, resolved_lsb)
        return DFPartselect(resolved_var, resolved_msb, resolved_lsb)

    if isinstance(tree, DFPointer):
        resolved_ptr = reorder(tree.ptr)
        resolved_var = reorder(tree.var)
        if isinstance(resolved_ptr, DFBranch):
            #raise FormatError('PTR should not be DFBranch')n
            return DFBranch(resolved_ptr.condnode, 
                            reorder(DFPointer(resolved_var, resolved_ptr.truenode)), 
                            reorder(DFPointer(resolved_var, resolved_ptr.falsenode)))
        if isinstance(resolved_var, DFBranch):
            return insertPointer(resolved_var, resolved_ptr)
        return DFPointer(resolved_var, resolved_ptr)

    if isinstance(tree, DFDelay):
        return DFDelay(reorder(tree.nextnode))

    raise DefinitionError('Undefined DFNode type: %s %s' % (str(type(tree)), str(tree)))

############################################################################
def insertBranch(base, truenode, falsenode):
    if isinstance(base, DFBranch):
        return DFBranch(base.condnode, insertBranch(base.truenode, truenode, falsenode), insertBranch(base.falsenode, truenode, falsenode))
    return DFBranch(base, truenode, falsenode)

def insertUnaryOp(base, op):
    if isinstance(base, DFBranch):
        return DFBranch(base.condnode, insertUnaryOp(base.truenode, op), insertUnaryOp(base.falsenode, op))
    return DFOperator((base,), op)

def insertOp(left, right, op):
    if isinstance(left, DFBranch):
        return DFBranch(left.condnode, insertOp(left.truenode, right, op), insertOp(left.falsenode, right, op))
    elif isinstance(right, DFBranch):
        return DFBranch(right.condnode, insertOp(left, right.truenode, op), insertOp(left, right.falsenode, op))
    return DFOperator((left, right), op)

def insertOpList(nextnodes, op):
    donenodes = []
    restnodes = list(nextnodes)
    for n in nextnodes:
        restnodes.pop(0)
        if isinstance(n, DFBranch):
            return DFBranch(n.condnode, insertOpList(tuple(donenodes + [n.truenode,] + restnodes), op), insertOpList(tuple(donenodes + [n.falsenode,] + restnodes), op))
        donenodes.append(n)
    return DFOperator(nextnodes, op)

def insertConcat(nextnodes):
    donenodes = []
    restnodes = list(nextnodes)
    for n in nextnodes:
        restnodes.pop(0)
        if isinstance(n, DFBranch):
            return DFBranch(n.condnode, insertConcat(tuple(donenodes + [n.truenode,] + restnodes)), insertConcat(tuple(donenodes + [n.falsenode,] + restnodes)))
        donenodes.append(n)
    return DFConcat(nextnodes)

def insertPartselect(var, msb, lsb):
    if isinstance(var, DFBranch):
        return DFBranch(var.condnode, insertPartselect(var.truenode, msb, lsb), insertPartselect(var.falsenode, msb, lsb))
    if var is None: return None
    return DFPartselect(var, msb, lsb)

def insertPointer(var, ptr):
    if isinstance(var, DFBranch):
        return DFBranch(var.condnode, insertPointer(var.truenode, ptr), insertPointer(var.falsenode, ptr))
    if var is None: return None
    return DFPointer(var, ptr)



#-------------------------------------------------------------------------------
# dataflow_analyzer.py
# 
# Verilog module signal/module dataflow analyzer
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os
import subprocess

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )
sys.setrecursionlimit(16 * 1024)

import pyverilog
import pyverilog.utils
import pyverilog.utils.version
from pyverilog.vparser.parser import VerilogCodeParser

if sys.version_info[0] >= 3:
    from pyverilog.dataflow.modulevisitor import ModuleVisitor
    from pyverilog.dataflow.signalvisitor import SignalVisitor
    from pyverilog.dataflow.bindvisitor import BindVisitor
else:
    from modulevisitor import ModuleVisitor
    from signalvisitor import SignalVisitor
    from bindvisitor import BindVisitor

class VerilogDataflowAnalyzer(VerilogCodeParser):
    def __init__(self, filelist, topmodule='TOP', noreorder=False, nobind=False,
                 preprocess_include=None,
                 preprocess_define=None):
        self.topmodule = topmodule
        self.terms = {}
        self.binddict = {}
        self.frametable = None
        files = filelist if isinstance(filelist, tuple) or isinstance(filelist, list) else [ filelist ]
        VerilogCodeParser.__init__(self, files,
                                   preprocess_include=preprocess_include,
                                   preprocess_define=preprocess_define)
        self.noreorder = noreorder
        self.nobind = nobind
        
    def generate(self):
        ast = self.parse()

        module_visitor = ModuleVisitor()
        module_visitor.visit(ast)
        modulenames = module_visitor.get_modulenames()
        moduleinfotable = module_visitor.get_moduleinfotable()
        
        signal_visitor = SignalVisitor(moduleinfotable, self.topmodule)
        signal_visitor.start_visit()
        frametable = signal_visitor.getFrameTable()

        if self.nobind:
            self.frametable = frametable
            return

        bind_visitor = BindVisitor(moduleinfotable, self.topmodule, frametable,
                                   noreorder=self.noreorder)

        bind_visitor.start_visit()
        dataflow = bind_visitor.getDataflows()

        self.frametable = bind_visitor.getFrameTable()
        self.terms = dataflow.getTerms()
        self.binddict = dataflow.getBinddict()

    def getFrameTable(self):
        return self.frametable

    #-------------------------------------------------------------------------
    def getInstances(self):
        if self.frametable is None: return ()
        return self.frametable.getAllInstances()

    def getSignals(self):
        if self.frametable is None: return ()
        return self.frametable.getAllSignals()

    def getConsts(self):
        if self.frametable is None: return ()
        return self.frametable.getAllConsts()

    def getTerms(self):
        return self.terms

    def getBinddict(self):
        return self.binddict

if __name__ == '__main__':
    from optparse import OptionParser
    INFO = "Verilog module signal/module dataflow analyzer"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python dataflow_analyzer.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-I","--include",dest="include",action="append",
                         default=[],help="Include path")
    optparser.add_option("-D",dest="define",action="append",
                         default=[],help="Macro Definition")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("--nobind",action="store_true",dest="nobind",
                         default=False,help="No binding traversal, Default=False")
    optparser.add_option("--noreorder",action="store_true",dest="noreorder",
                         default=False,help="No reordering of binding dataflow, Default=False")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    verilogdataflowanalyzer = VerilogDataflowAnalyzer(filelist, options.topmodule,
                                                      noreorder=options.noreorder,
                                                      nobind=options.nobind,
                                                      preprocess_include=options.include,
                                                      preprocess_define=options.define)
    verilogdataflowanalyzer.generate()

    directives = verilogdataflowanalyzer.get_directives()
    print('Directive:')
    for dr in sorted(directives, key=lambda x:str(x)):
        print(dr)

    instances = verilogdataflowanalyzer.getInstances()
    print('Instance:')
    for module, instname in sorted(instances, key=lambda x:str(x[1])):
        print((module, instname))

    if options.nobind:
        print('Signal:')
        signals = verilogdataflowanalyzer.getSignals()
        for sig in signals:
            print(sig)

        print('Const:')
        consts = verilogdataflowanalyzer.getConsts()
        for con in consts:
            print(con)

    else:
        terms = verilogdataflowanalyzer.getTerms()
        print('Term:')
        for tk, tv in sorted(terms.items(), key=lambda x:str(x[0])):
            print(tv.tostr())
   
        binddict = verilogdataflowanalyzer.getBinddict()
        print('Bind:')
        for bk, bv in sorted(binddict.items(), key=lambda x:str(x[0])):
            for bvi in bv:
                print(bvi.tostr())



#-------------------------------------------------------------------------------
# codegen.py
#
# Code generator from dataflow
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os
import re

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

import pyverilog.utils.version
import pyverilog.utils.util as util
import pyverilog.utils.verror as verror
import pyverilog.utils.signaltype as signaltype
from pyverilog.utils.scope import ScopeLabel, ScopeChain

if sys.version_info[0] >= 3:
    from pyverilog.dataflow.subset import VerilogSubset
else:
    from subset import VerilogSubset

class VerilogCodeGenerator(VerilogSubset):
    def __init__(self, topmodule, terms, binddict, 
                 resolved_terms, resolved_binddict, constlist,
                 modulename='Subset', enable_name='HT_enable', num_indent=2, flat=True):
        VerilogSubset.__init__(self, topmodule, terms, binddict, 
                               resolved_terms, resolved_binddict, constlist)
        self.modulename = modulename
        self.enable_name = enable_name
        self.num_indent = num_indent
        self.flat = flat

    ############################################################################
    def addBind(self, name, bind):
        if name in self.binddict:
            self.binddict[name] += (bind,)
        else:
            self.binddict[name] = (bind,)
        if name in self.resolved_binddict:
            self.resolved_binddict[name] += (bind,)
        else:
            self.resolved_binddict[name] = (bind,)

    def addTerm(self, name, term):
        self.terms[name] = term
        self.resolved_terms[name] = term

    ############################################################################
    def _input(self, name):
        return 'input ' + name + ';\n'
    def _output(self, name):
        return 'output ' + name + ';\n'
    def _inout(self, name):
        return 'inout ' + name + ';\n'
    def _wire(self, name):
        return 'wire ' + name + ';\n'

    ############################################################################
    def _modulehead(self, terms):
        modulehead = ''
        modulehead += '`default_nettype none\n'
        modulehead += 'module ' + self.modulename + '('
        modulehead += '\n' + (' ' * self.num_indent)
        modulehead += self.clock_name + ', '
        modulehead += self.reset_name + ', '
        modulehead += self.enable_name + ', '
        modulehead += '\n' + (' ' * self.num_indent)
        
        for tk, tv in terms.items():
            scope = util.getScope(tk)
            if util.isTopmodule(scope):
                termtype = self.getTermtype(tk)
                if signaltype.isInput(termtype) and tk == util.toTermname((self.topmodule, self.reset_name)): continue
                if signaltype.isInput(termtype) and tk == util.toTermname((self.topmodule, self.clock_name)): continue
                if signaltype.isInput(termtype): modulehead += util.toFlatname(tk) + ', '
                elif signaltype.isInout(termtype): modulehead += util.toFlatname(tk) + ', '
                elif signaltype.isOutput(termtype): modulehead += util.toFlatname(tk) + ', '
        modulehead = modulehead[:-2] + ');\n\n'
        return modulehead

    ############################################################################
    def _system_io(self, clock_name, reset_name, enable_name):
        flat_clock_name = util.toFlatname( util.toTermname((self.topmodule, clock_name)) )
        flat_reset_name = util.toFlatname( util.toTermname((self.topmodule, reset_name)) )
        flat_enable_name = util.toFlatname( util.toTermname((self.topmodule, enable_name)) )
        code = ''
        code += self._input(clock_name)
        code += self._input(reset_name)
        code += self._input(enable_name)
        code += self._wire(flat_clock_name)
        code += self._wire(flat_reset_name)
        code += self._assign(flat_clock_name, clock_name)
        code += self._assign(flat_reset_name, reset_name)
        code += '\n'
        return code

    ############################################################################
    def _assign(self, left, right, msb=None, lsb=None):
        code = 'assign ' + left 
        if msb is not None and lsb is not None:
            code += '[' + msb.tocode(None)  + ':' + lsb.tocode(None) + ']'
        elif msb is not None:
            code += '[' + msb.tocode(None)  + ']'
        elif lsb is not None:
            code += '[' + lsb.tocode(None)  + ']'
        code += ' = ' + right + ';\n'
        return code

    ############################################################################
    def generateCode(self, targets=()):
        if len(targets) > 0:
            return self.generateSubsetCode(targets)
        return self.generateEntireCode()

    def generateSubsetCode(self, targets):
        terms, parameter, assign, always_clockedge, always_combination = self.getSubset(targets)
        return self._toCode(terms, parameter, assign, always_clockedge, always_combination)

    def generateEntireCode(self):
        terms, parameter, assign, always_clockedge, always_combination = self.getEntire()
        return self._toCode(terms, parameter, assign, always_clockedge, always_combination)

    def _toCode(self, terms, parameter, assign, always_clockedge, always_combination):
        # module header
        modulehead = self._modulehead(terms)

        code = ''
        # clock, reset, control input definition
        code += self._system_io(self.clock_name, self.reset_name, self.enable_name)

        # general signal definition
        for tk, tv in terms.items():
            termtype = self.getTermtype(tk)
            if signaltype.isInput(termtype) and tk == util.toTermname((self.topmodule, self.reset_name)): continue
            if signaltype.isInput(termtype) and tk == util.toTermname((self.topmodule, self.clock_name)): continue
            code += tv.tocode()

        for pk, pv in parameter.items():
            code += pv.tocode()

        # assign
        for ak, avv in assign.items():
            cnt = 0
            for av in avv:
                code += av.tocode()

        # always (clock edge)
        for ck, cvv in always_clockedge.items():
            for cv in cvv:
                code += cv.tocode()

        # always (combination)
        for ck, cvv in always_combination.items():
            for cv in cvv:
                code += cv.tocode()
            
        # module tail
        moduletail = '\nendmodule\n'

        ret = modulehead + code + moduletail
        
        if self.flat:
            ret = re.sub('\.', '_', ret)

        return ret

################################################################################
if __name__ == '__main__':
    from optparse import OptionParser
    import pyverilog.utils.util as util
    if sys.version_info[0] >= 3:
        from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
        from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
    else:
        from dataflow_analyzer import VerilogDataflowAnalyzer
        from optimizer import VerilogDataflowOptimizer
    INFO = "Code generator from Verilog dataflow definitions"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python codegen.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-I","--include",dest="include",action="append",
                         default=[],help="Include path")
    optparser.add_option("-D",dest="define",action="append",
                         default=[],help="Macro Definition")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("--nobind",action="store_true",dest="nobind",
                         default=False,help="No binding traversal, Default=False")
    optparser.add_option("--noreorder",action="store_true",dest="noreorder",
                         default=False,help="No reordering of binding dataflow, Default=False")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    optparser.add_option("-o","--output",dest="outputfile",
                         default="helperthread.v",help="Output File name, Default=helperthread.v")
    optparser.add_option("--clockname",dest="clockname",
                         default="CLK",help="Clock signal name")
    optparser.add_option("--resetname",dest="resetname",
                         default="RST_X",help="Reset signal name")
    optparser.add_option("--clockedge",dest="clockedge",
                         default="posedge",help="Clock signal edge")
    optparser.add_option("--resetedge",dest="resetedge",
                         default="negedge",help="Reset signal edge")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule,
                                       noreorder=options.noreorder,
                                       nobind=options.nobind,
                                       preprocess_include=options.include,
                                       preprocess_define=options.define)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)

    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()

    codegen = VerilogCodeGenerator(options.topmodule, terms, binddict,
                                   resolved_terms, resolved_binddict, constlist)
    codegen.set_clock_info(options.clockname, options.clockedge)
    codegen.set_reset_info(options.resetname, options.resetedge)
    code = codegen.generateCode(options.searchtarget)

    f = open(options.outputfile, 'w')
    f.write(code)
    f.close()



#-------------------------------------------------------------------------------
# signaltype.py
# 
# Signal type checker to create precise assignment tree
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import re

################################################################################
# Verilog Signal Type Checker
################################################################################
def isType(termtype, matchtype):
    for t in termtype:
        if t == matchtype: return True
    return False
def isInput(termtype):
    return isType(termtype, 'Input')
def isOutput(termtype):
    return isType(termtype, 'Output')
def isInout(termtype):
    return isType(termtype, 'Inout')
def isWire(termtype):
    return isType(termtype, 'Wire')
def isReg(termtype):
    return isType(termtype, 'Reg')
def isWireArray(termtype):
    return isType(termtype, 'WireArray')
def isRegArray(termtype):
    return isType(termtype, 'RegArray')
def isInteger(termtype):
    return isType(termtype, 'Integer')
def isGenvar(termtype):
    return isType(termtype, 'Genvar')
def isParameter(termtype):
    return isType(termtype, 'Parameter')
def isLocalparam(termtype):
    return isType(termtype, 'Localparam')
def isFunction(termtype):
    return isType(termtype, 'Function')
def isRename(termtype):
    return isType(termtype, 'Rename')
def isArray(termtype):
    if isWireArray(termtype): return True
    if isRegArray(termtype): return True
    return False

################################################################################
# Clock/Reset
################################################################################
regex_clock = ['clk', 'clock', ]
regex_reset = ['reset', 'rst', ]
def isClock(search_str): 
    lower_str = search_str.lower()
    for rc in regex_clock:
        if re.search(rc, lower_str):
            return True
    return False
def isReset(search_str): 
    lower_str = search_str.lower()
    for rr in regex_reset:
        if re.search(rr, lower_str):
            return True
    return False

################################################################################
# Operator
################################################################################
compare_ops = ('LessThan', 'GreaterThan', 'LassEq', 'GreaterEq', 'Eq', 'NotEq', 'Eql', 'NotEql')
not_ops = ('Ulnot', 'Unot')
split_and_ops = ('And', 'Land') 
split_or_ops = ('Or', 'Lor')
non_condition_ops = compare_ops + not_ops + split_and_ops + split_or_ops

def isCompare(op):
    if op in compare_ops: return True
    return False
def isNot(op):
    if op in not_ops: return True
    return False
def isAnd(op):
    if op in split_and_ops: return True
    return False
def isOr(op):
    if op in split_or_ops: return True
    return False
def isNonConditionOp(op):
    if op in non_condition_ops: return False
    return True



#-------------------------------------------------------------------------------
# walker.py
#
# Dataflow graph walker
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

import pyverilog.utils.version
import pyverilog.utils.util as util
import pyverilog.utils.verror as verror
import pyverilog.utils.signaltype as signaltype

if sys.version_info[0] >= 3:
    from pyverilog.dataflow.dataflow import *
    from pyverilog.dataflow.visit import *
    from pyverilog.dataflow.optimizer import VerilogOptimizer
    from pyverilog.dataflow.merge import VerilogDataflowMerge
    import pyverilog.dataflow.reorder as reorder
    import pyverilog.dataflow.replace as replace
else:
    from dataflow import *
    from visit import *
    from optimizer import VerilogOptimizer
    from merge import VerilogDataflowMerge
    import reorder
    import replace

class VerilogDataflowWalker(VerilogDataflowMerge):
    def __init__(self, topmodule, terms, binddict, resolved_terms, resolved_binddict, constlist):
        VerilogDataflowMerge.__init__(self, topmodule, terms, binddict,
                                      resolved_terms, resolved_binddict, constlist)

    ############################################################################
    def walkBind(self, name, step=0):
        termname = util.toTermname(name)
        if not termname in self.terms: raise verror.DefinitionError('No such signals: %s' % str(name))
        tree = self.getTree(termname)
        walked_tree = self.walkTree(tree, visited=set(), step=step)
        return replace.replaceUndefined(walked_tree, termname)

    ############################################################################
    def walkTree(self, tree, visited=set([]), step=0, delay=False, msb=None, lsb=None, ptr=None):
        if tree is None:
            return DFUndefined(32)

        if isinstance(tree, DFUndefined):
            return tree

        if isinstance(tree, DFHighImpedance):
            return tree

        if isinstance(tree, DFConstant):
            return tree

        if isinstance(tree, DFEvalValue):
            return tree

        if isinstance(tree, DFTerminal):
            scope = util.getScope(tree.name)
            termname = tree.name
            if termname in visited: return tree

            termtype = self.getTermtype(termname)
            if util.isTopmodule(scope) and signaltype.isInput(termtype):
                return tree

            nptr = None
            if signaltype.isRegArray(termtype) or signaltype.isWireArray(termtype):
                if ptr is None:
                    raise verror.FormatError('Array variable requires an pointer.')
                if msb is not None and lsb is not None: return tree
                nptr = ptr

            nextstep = step
            if signaltype.isReg(termtype) or signaltype.isRegArray(termtype):
                if (not self.isCombination(termname) and
                    not signaltype.isRename(termtype) and 
                    nextstep == 0):
                    return tree
                if (not self.isCombination(termname) and
                    not signaltype.isRename(termtype)):
                    nextstep -= 1

            return self.walkTree(self.getTree(termname, nptr),
                                 visited|set([termname,]), nextstep, delay)

        if isinstance(tree, DFBranch):
            condnode = self.walkTree(tree.condnode, visited, step, delay)
            truenode = self.walkTree(tree.truenode, visited, step, delay)
            falsenode = self.walkTree(tree.falsenode, visited, step, delay)
            return DFBranch(condnode, truenode, falsenode)

        if isinstance(tree, DFOperator):
            nextnodes = []
            for n in tree.nextnodes:
                nextnodes.append(self.walkTree(n, visited, step, delay))
            return DFOperator(tuple(nextnodes), tree.operator)

        if isinstance(tree, DFPartselect):
            msb = self.walkTree(tree.msb, visited, step, delay)
            lsb = self.walkTree(tree.lsb, visited, step, delay)
            var = self.walkTree(tree.var, visited, step, delay, msb=msb, lsb=lsb)
            return DFPartselect(var, msb, lsb)

        if isinstance(tree, DFPointer):
            ptr = self.walkTree(tree.ptr, visited, step, delay)
            var = self.walkTree(tree.var, visited, step, delay, ptr=ptr)
            if isinstance(tree.var, DFTerminal):
                termtype = self.getTermtype(tree.var.name)
                if ((signaltype.isRegArray(termtype) or 
                     signaltype.isWireArray(termtype)) and 
                    not (isinstance(var, DFTerminal) and var.name == tree.var.name)):
                    return var
            return DFPointer(var, ptr)

        if isinstance(tree, DFConcat):
            nextnodes = []
            for n in tree.nextnodes:
                nextnodes.append(self.walkTree(n, visited, step, delay))
            return DFConcat(tuple(nextnodes))

        raise verror.DefinitionError(
            'Undefined Node Type: %s : %s' % (str(type(tree)), str(tree)))

################################################################################
if __name__ == '__main__':
    from optparse import OptionParser
    import pyverilog.utils.util as util
    if sys.version_info[0] >= 3:
        from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
        from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
    else:
        from dataflow_analyzer import VerilogDataflowAnalyzer
        from optimizer import VerilogDataflowOptimizer
    INFO = "Dataflow walker"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python walker.py -t TOPMODULE -s TARGETSIGNAL file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-I","--include",dest="include",action="append",
                         default=[],help="Include path")
    optparser.add_option("-D",dest="define",action="append",
                         default=[],help="Macro Definition")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("--nobind",action="store_true",dest="nobind",
                         default=False,help="No binding traversal, Default=False")
    optparser.add_option("--noreorder",action="store_true",dest="noreorder",
                         default=False,help="No reordering of binding dataflow, Default=False")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule,
                                       noreorder=options.noreorder,
                                       nobind=options.nobind,
                                       preprocess_include=options.include,
                                       preprocess_define=options.define)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)

    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()

    walker = VerilogDataflowWalker(options.topmodule, terms, binddict, resolved_terms,
                                   resolved_binddict, constlist)

    for target in options.searchtarget:
        tree = walker.walkBind(target)
        print('target: %s' % target)
        print(tree.tostr())



#-------------------------------------------------------------------------------
# active_analyzer.py
#
# Active condition list generator from Verilog Definitions
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

import pyverilog.utils.version
import pyverilog.utils.util as util
import pyverilog.utils.inference as inference
import pyverilog.dataflow.reorder as reorder
from pyverilog.dataflow.dataflow import *

if sys.version_info[0] >= 3:
    import pyverilog.controlflow.splitter as splitter
    import pyverilog.controlflow.transition as transition
    from pyverilog.controlflow.controlflow_analyzer import VerilogControlflowAnalyzer
else:
    import splitter
    import transition
    from controlflow_analyzer import VerilogControlflowAnalyzer

class VerilogActiveConditionAnalyzer(VerilogControlflowAnalyzer):
    def __init__(self, topmodule, terms, binddict, 
                 resolved_terms, resolved_binddict, constlist):
        VerilogControlflowAnalyzer.__init__(self, topmodule, terms, binddict, 
                                            resolved_terms, resolved_binddict, constlist)
        self.fsm_loops, self.fsms = self.getLoops()

    ############################################################################
    def getActiveConditions(self, termname, condition=splitter.active_constant):
        if not termname in self.resolved_binddict: return {}
        tree = self.makeTree(termname)
        funcdict = splitter.split(tree)
        funcdict = splitter.filter(funcdict, termname, condition)
        funcdict = splitter.remove_reset_condition(funcdict)

        if len(funcdict) == 1 and len(list(funcdict.keys())[0]) == 0:
            func = funcdict.values()[0]
            return {termname : ( ('any', None), )}

        active_conditions = {}
        active_conditions_size = 0
        for fsm_sig in self.fsms.keys():
            rslt = self.getActiveConditions_fsm(fsm_sig, funcdict)
            if len(rslt) > 0: active_conditions[fsm_sig] = rslt
            active_conditions_size += len(rslt)

        if active_conditions_size == 0:
            rslt = self.getActiveConditions_fsm(termname, funcdict)
            if len(rslt) > 0: active_conditions[termname] = rslt

        return active_conditions

    def getActiveConditions_fsm(self, fsm_sig, funcdict):
        # returns a list of some (state, transcond) pairs
        active_conditions = []
        fsm_sig_width = self.getWidth(fsm_sig)
        for condlist, func in sorted(funcdict.items(), key=lambda x:len(x[0])):
            node = transition.walkCondlist(condlist, fsm_sig, fsm_sig_width)
            state_node_list = []
            if isinstance(node, transition.StateNodeList):
                for n in node.nodelist: state_node_list.append(n)
            elif node:
                state_node_list.append(node)

            for state_node in state_node_list:
                #if state_node.isany:
                #    active_conditions.append( ('any', state_node.transcond) )
                for rs, re in state_node.range_pairs:
                    for state in range(rs, re+1):
                        transcond = self.optimizer.optimize(state_node.transcond)
                        if isinstance(transcond, DFEvalValue) and transcond.value == 0: continue
                        active_conditions.append( (state, transcond) )
        return tuple(active_conditions)
            
################################################################################
if __name__ == '__main__':
    from optparse import OptionParser
    import pyverilog.utils.util as util
    from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
    from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
    INFO = "Active condition analyzer"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python active_analyzer.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)

    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()

    canalyzer = VerilogActiveConditionAnalyzer(options.topmodule, terms, binddict, 
                                               resolved_terms, resolved_binddict, constlist)

    for target in options.searchtarget:
        signal = util.toTermname(target)

        active_conditions = canalyzer.getActiveConditions( signal )
        #active_conditions = canalyzer.getActiveConditions( signal, condition=splitter.active_modify )
        #active_conditions = canalyzer.getActiveConditions( signal, condition=splitter.active_unmodify )

        print('Active Cases: %s' % signal)
        for fsm_sig, active_conditions in sorted(active_conditions.items(), key=lambda x:str(x[0])):
            print('FSM: %s' % fsm_sig)
            for state, active_condition in sorted(active_conditions, key=lambda x:str(x[0])):
                s = []
                s.append('state: %d -> ' % state)
                if active_condition: s.append(active_condition.tocode())
                else: s.append('empty')
                print(''.join(s))



#-------------------------------------------------------------------------------
# subset.py
#
# Generating Subset of dataflow graph
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os
import collections

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

import pyverilog.utils.version
import pyverilog.utils.util as util
import pyverilog.utils.verror as verror
import pyverilog.utils.signaltype as signaltype

if sys.version_info[0] >= 3:
    from pyverilog.dataflow.merge import VerilogDataflowMerge
else:
    from merge import VerilogDataflowMerge

class VerilogSubset(VerilogDataflowMerge):
    def __init__(self, topmodule, terms, binddict,
                 resolved_terms, resolved_binddict, constlist):
        VerilogDataflowMerge.__init__(self, topmodule, terms, binddict,
                                      resolved_terms, resolved_binddict, constlist)
        self.clock_name = 'CLK'
        self.clock_edge = 'posedge'
        self.reset_name = 'RST_X'
        self.reset_edge = 'negedge'

    ############################################################################
    def set_clock_info(self, clock_name, clock_edge):
        self.clock_name = clock_name
        self.clock_edge = clock_edge

    def set_reset_info(self, reset_name, reset_edge):
        self.reset_name = reset_name
        self.reset_edge = reset_edge

    ############################################################################
    def getBindSubset(self, termname, visited_sources=set()):
        term = self.getTerm(termname)
        if term is None: raise verror.DefinitionError('No such signal')
        bindlist = self.getBindlist(termname)
        nextsources = visited_sources.copy()
        ret_binds = collections.OrderedDict()
        for bind in bindlist:
            if not termname in ret_binds:
                ret_binds[termname] = []
            ret_binds[termname].append(bind)
            if bind.isClockEdge():
                clock_name = bind.getClockName()
                if clock_name != util.toTermname((self.topmodule, self.clock_name)):
                    r_binds, r_sources = self.getBindSubset(clock_name, nextsources)
                    nextsources |= r_sources
                    ret_binds = util.dictlistmerge(ret_binds, r_binds)

        sources = self.getBindSources(termname)
        for source in sources:
            if source in visited_sources: continue
            nextsources.add(source)
            r_binds, r_sources = self.getBindSubset(source, nextsources)
            ret_binds = util.dictlistmerge(ret_binds, r_binds)
            nextsources |= r_sources
        return ret_binds, nextsources

    ############################################################################
    def getBindSourceSubset(self, targets):
        visited_binddict = collections.OrderedDict()
        visited_sources = set()
        for target in targets:
            termname = util.toTermname(target)
            r_binds, r_sources = self.getBindSubset(termname, visited_sources)
            visited_sources |= r_sources
            visited_binddict = util.dictlistmerge(visited_binddict, r_binds)
        return visited_binddict, visited_sources

    ############################################################################
    def getEntire(self):
        visited_binddict = self.resolved_binddict
        visited_sources = self.terms.keys()
        return self._discretion(visited_binddict, visited_sources)

    def getSubset(self, targets):
        visited_binddict, visited_sources = self.getBindSourceSubset(targets)
        return self._discretion(visited_binddict, visited_sources)

    def _discretion(self, visited_binddict, visited_sources):
        terms = collections.OrderedDict()
        parameter = collections.OrderedDict()
        assign = collections.OrderedDict()
        always_clockedge = collections.OrderedDict()
        always_combination = collections.OrderedDict()

        # discretion the signal desclaration
        for source in visited_sources:
            termtype = self.getTermtype(source)
            if signaltype.isParameter(termtype): continue
            if signaltype.isLocalparam(termtype): continue
            terms[source] = self.terms[source]

        for left, rights in visited_binddict.items():
            for right in rights:
                assign_type = self.getAssignType(left, right)
                if assign_type == 'clockedge':
                    if not left in always_clockedge: always_clockedge[left] = set([])
                    always_clockedge[left].add(right)
                elif assign_type == 'combination':
                    if not left in always_combination: always_combination[left] = set([])
                    always_combination[left].add(right)
                elif assign_type == 'assign':
                    if not left in assign: assign[left] = set([])
                    assign[left].add(right)
                elif assign_type == 'parameter':
                    parameter[left] = right
                    continue
                elif assign_type == 'localparam':
                    parameter[left] = right
                    continue
                if not left in terms: terms[left] = self.terms[left]
        return terms, parameter, assign, always_clockedge, always_combination

################################################################################
if __name__ == '__main__':
    from optparse import OptionParser
    import pyverilog.utils.util as util
    if sys.version_info[0] >= 3:
        from pyverilog.dataflow.dataflow_analyzer import VerilogDataflowAnalyzer
        from pyverilog.dataflow.optimizer import VerilogDataflowOptimizer
    else:
        from dataflow_analyzer import VerilogDataflowAnalyzer
        from optimizer import VerilogDataflowOptimizer
    INFO = "Subset generator from Verilog dataflow definitions"
    VERSION = pyverilog.utils.version.VERSION
    USAGE = "Usage: python subset.py -t TOPMODULE file ..."

    def showVersion():
        print(INFO)
        print(VERSION)
        print(USAGE)
        sys.exit()
    
    optparser = OptionParser()
    optparser.add_option("-v","--version",action="store_true",dest="showversion",
                         default=False,help="Show the version")
    optparser.add_option("-I","--include",dest="include",action="append",
                         default=[],help="Include path")
    optparser.add_option("-D",dest="define",action="append",
                         default=[],help="Macro Definition")
    optparser.add_option("-t","--top",dest="topmodule",
                         default="TOP",help="Top module, Default=TOP")
    optparser.add_option("--nobind",action="store_true",dest="nobind",
                         default=False,help="No binding traversal, Default=False")
    optparser.add_option("--noreorder",action="store_true",dest="noreorder",
                         default=False,help="No reordering of binding dataflow, Default=False")
    optparser.add_option("-s","--search",dest="searchtarget",action="append",
                         default=[],help="Search Target Signal")
    optparser.add_option("--clockname",dest="clockname",
                         default="CLK",help="Clock signal name")
    optparser.add_option("--resetname",dest="resetname",
                         default="RST_X",help="Reset signal name")
    optparser.add_option("--clockedge",dest="clockedge",
                         default="posedge",help="Clock signal edge")
    optparser.add_option("--resetedge",dest="resetedge",
                         default="negedge",help="Reset signal edge")
    (options, args) = optparser.parse_args()

    filelist = args
    if options.showversion:
        showVersion()

    for f in filelist:
        if not os.path.exists(f): raise IOError("file not found: " + f)

    if len(filelist) == 0:
        showVersion()

    analyzer = VerilogDataflowAnalyzer(filelist, options.topmodule,
                                       noreorder=options.noreorder,
                                       nobind=options.nobind,
                                       preprocess_include=options.include,
                                       preprocess_define=options.define)
    analyzer.generate()

    directives = analyzer.get_directives()
    terms = analyzer.getTerms()
    binddict = analyzer.getBinddict()

    optimizer = VerilogDataflowOptimizer(terms, binddict)

    optimizer.resolveConstant()
    resolved_terms = optimizer.getResolvedTerms()
    resolved_binddict = optimizer.getResolvedBinddict()
    constlist = optimizer.getConstlist()

    subset = VerilogSubset(options.topmodule, terms, binddict, 
                           resolved_terms, resolved_binddict, constlist)

    subset.set_clock_info(options.clockname, options.clockedge)
    subset.set_reset_info(options.resetname, options.resetedge)

    sub_binds, sub_terms = subset.getBindSourceSubset(options.searchtarget)
    terms, parameter, assign, always_clockedge, always_combination = subset.getSubset(options.searchtarget)

    for k, v in terms.items():
        print(v.tocode())
    for k, v in parameter.items():
        print(v.tocode())
    for k, v in assign.items():
        for vv in v:
            print(vv.tocode())
    for k, v in always_clockedge.items():
        for vv in v:
            print(vv.tocode())
    for k, v in always_combination.items():
        for vv in v:
            print(vv.tocode())



#-------------------------------------------------------------------------------
# transition.py
#
# Splitting a tree into state part and transition condition part
#
# Copyright (C) 2013, Shinya Takamaeda-Yamazaki
# License: Apache 2.0
#-------------------------------------------------------------------------------

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) )

from pyverilog.dataflow.dataflow import *
import pyverilog.utils.util as util
import pyverilog.utils.signaltype as signaltype
import pyverilog.utils.inference as inference

def walkCondlist(condlist, termname, termwidth=32):
    node = None
    if len(condlist) == 0: 
        maxvalue = util.maxValue(termwidth)
        return StateNode(maxvalue=maxvalue, isany=True)
    for cond in condlist:
        rslt = walkCond(cond, termname, termwidth)
        if node and rslt: node = andStateNodeList(node, rslt)
        elif rslt: node = rslt
    return node

############################################################################
def walkCond(cond, termname, termwidth=32):
    if isinstance(cond, DFOperator):
        if len(cond.nextnodes) == 1:
            return _walkCond_DFOperator_unary(cond, termname, termwidth)
        if len(cond.nextnodes) == 2:
            return _walkCond_DFOperator_dual(cond, termname, termwidth)
    if isinstance(cond, DFTerminal):
        comp_cond = DFOperator((cond, DFEvalValue(0)), 'GreaterThan')
        return _walkCond_DFOperator_dual(comp_cond, termname, termwidth)
    maxvalue = util.maxValue(termwidth)
    return StateNode(transcond=cond, maxvalue=maxvalue, isany=True)

def _walkCond_DFOperator_unary(cond, termname, termwidth=32):
    node = walkCond(cond.nextnodes[0], termname, termwidth)
    if signaltype.isNot(cond.operator): node = notStateNodeList(node)
    return node

def _walkCond_DFOperator_dual(cond, termname, termwidth=32):
    maxvalue = util.maxValue(termwidth)
    if signaltype.isNonConditionOp(cond.operator):
        return StateNode(transcond=cond, maxvalue=maxvalue, isany=True)

    if not signaltype.isCompare(cond.operator) and signaltype.isOr(cond.operator):
        lnode = walkCond(cond.nextnodes[0], termname, termwidth)
        rnode = walkCond(cond.nextnodes[1], termname, termwidth)
        ret = orStateNodeList(lnode, rnode)
        if ret: return ret
    if not signaltype.isCompare(cond.operator) and signaltype.isAnd(cond.operator):
        lnode = walkCond(cond.nextnodes[0], termname, termwidth)
        rnode = walkCond(cond.nextnodes[1], termname, termwidth)
        ret = andStateNodeList(lnode, rnode)
        if ret: return ret

    l = cond.nextnodes[0]
    r = cond.nextnodes[1]

    if isinstance(l, DFTerminal) and l.name == termname:
        infval = inference.infer(cond.operator, r)
        return createStateNode(infval, termwidth)
    if isinstance(r, DFTerminal) and r.name == termname:
        new_op = re.sub('Greater', 'TMP', cond.operator)
        new_op = re.sub('Less', 'Greater', new_op)
        new_op = re.sub('TMP', 'Less', new_op)
        infval = inference.infer(new_op, l)
        return createStateNode(infval, termwidth)

    maxvalue = util.maxValue(termwidth)
    return StateNode(transcond=cond, maxvalue=maxvalue, isany=True)

############################################################################
def notStateNodeList(node):
    if isStateNode(node):
        return notStateNode(node)
    if isStateNodeList(node):
        new_nodes = []
        for n in node.nodelist:
            new_nodes.append(notStateNode(n))
        ret_node = None
        for n in new_nodes:
            if ret_node is None: ret_node = n
            else: ret_node = andStateNodeList(ret_node, n)
        return ret_node
    return None

def orStateNodeList(lnode, rnode):
    if isStateNode(lnode) and isStateNode(rnode):
        return StateNodeList( (lnode, rnode) )
    if isStateNodeList(lnode) and isStateNodeList(rnode):
        new_list = []
        new_list.extend(lnode.nodelist)
        new_list.extend(rnode.nodelist)
        return StateNodeList(tuple(new_list))
    if isStateNodeList(lnode) and isStateNode(rnode):
        new_list = []
        new_list.extend(lnode.nodelist)
        new_list.append(rnode)
        return StateNodeList(tuple(new_list))
    if isStateNode(lnode) and isStateNodeList(rnode):
        new_list = []
        new_list.append(lnode)
        new_list.extend(rnode.nodelist)
        return StateNodeList(tuple(new_list))
    return None

def andStateNodeList(lnode, rnode):
    if lnode is None and rnode is None: return None
    if lnode is None: return rnode
    if rnode is None: return lnode
    
    if isStateNode(lnode) and isStateNode(rnode):
        return andStateNode(lnode, rnode)
    if isStateNodeList(lnode) and isStateNodeList(rnode):
        new_list = []
        for l in lnode.nodelist:
            for r in rnode.nodelist:
                val = andStateNode(l, r)
                if val: new_list.append(val)
        if len(new_list) == 0: return None
        return StateNodeList(tuple(new_list))
    if isStateNodeList(lnode) and isStateNode(rnode):
        new_list = []
        for l in lnode.nodelist:
            val = andStateNode(l, rnode)
            if val: new_list.append(val)
        if len(new_list) == 0: return None
        return StateNodeList(tuple(new_list))
    if isStateNode(lnode) and isStateNodeList(rnode):
        new_list = []
        for r in rnode.nodelist:
            val = andStateNode(lnode, r)
            if val: new_list.append(val)
        if len(new_list) == 0: return None
        return StateNodeList(tuple(new_list))
    return None

def notStateNode(node):
    if node.isany:
        new_transcond = _not_transcond(node.transcond)
        return StateNode(transcond=new_transcond, maxvalue=node.maxvalue, isany=True)
    new_range_pairs = _not_range_pairs(node)
    a = StateNode(range_pairs=new_range_pairs, maxvalue=node.maxvalue)
    if node.transcond is None: return a
    new_transcond = _not_transcond(node.transcond)
    b = StateNode(range_pairs=node.range_pairs, transcond=new_transcond, maxvalue=node.maxvalue)
    return StateNodeList((a, b))

def andStateNode(sna, snb):
    if sna is None and snb is None: return None
    if sna is None: return snb
    if snb is None: return sna

    isany = False
    range_pairs = []
    if sna.isany and snb.isany:
        isany = True
    elif sna.isany:
        range_pairs = snb.range_pairs
    elif snb.isany:
        range_pairs = sna.range_pairs
    elif len(sna.range_pairs) > 0 and len(snb.range_pairs) > 0:
        range_pairs = _and_range_pairs(sna, snb)

    if (not isany) and len(range_pairs) == 0:
        return None

    transcond = None
    if isinstance(sna.transcond, DFNode) and isinstance(snb.transcond, DFNode):
        transcond = DFOperator((sna.transcond, snb.transcond),'Land')
    elif isinstance(sna.transcond, DFNode):
        transcond = sna.transcond
    elif isinstance(snb.transcond, DFNode):
        transcond = snb.transcond

    maxvalue = max(sna.maxvalue, snb.maxvalue)
    return StateNode(tuple(range_pairs), maxvalue=maxvalue, transcond=transcond, isany=isany)

def _and_range_pairs(sna, snb):
    range_pairs = []
    a_ptr = 0
    b_ptr = 0
    sorted_sna = sorted(sna.range_pairs, key=lambda x:x[0])
    sorted_snb = sorted(snb.range_pairs, key=lambda x:x[0])
    amin, amax = sorted_sna[a_ptr]
    bmin, bmax = sorted_snb[b_ptr]
    last_a_ptr = -1
    last_b_ptr = -1
    while True:
        next_min = max(amin, bmin)
        next_max = min(amax, bmax)
        if bmax < amin or bmin > amax: pass
        else: range_pairs.append( (next_min, next_max) )
        if amax >= bmax and b_ptr < len(sorted_snb)-1:
            b_ptr += 1
            bmin, bmax = sorted_snb[b_ptr]
        elif amax <= bmax and a_ptr < len(sorted_sna)-1:
            a_ptr += 1
            amin, amax = sorted_sna[a_ptr]
        if a_ptr == last_a_ptr and b_ptr == last_b_ptr: break
        last_a_ptr = a_ptr
        last_b_ptr = b_ptr
    ret_set = set(range_pairs)
    ret_list = list(ret_set)
    return ret_list

def _not_range_pairs(node):
    new_range_pairs = []
    cur_min = node.minvalue
    for rmin, rmax in sorted(node.range_pairs, key=lambda x:x[0]):
        if rmin-1 >= cur_min:
            new_range_pairs.append( (cur_min, rmin-1) )
        cur_min = max(rmax+1, cur_min)
    if cur_min <= node.maxvalue:
        new_range_pairs.append( (cur_min, node.maxvalue) )
    return new_range_pairs

def _not_transcond(transcond):
    if transcond is None:
        return None
    elif isinstance(transcond, DFOperator) and transcond.operator == 'Ulnot':
        return transcond.nextnodes[0]
    elif isinstance(transcond, DFOperator) and transcond.operator == 'Unot':
        return transcond.nextnodes[0]
    return DFOperator((transcond,), 'Ulnot')

def createStateNode(infval, width):
    statelabels = []
    minval = infval.minval
    maxval = infval.maxval
    if minval is None: minval = 0
    if maxval is None: maxval = util.maxValue(width)
    ret = StateNode( ((minval, maxval),), 0, util.maxValue(width), None)
    if infval.inv: ret = notStateNodeList(ret)
    return ret

def isStateNode(node):
    return isinstance(node, StateNode)

def isStateNodeList(node):
    return isinstance(node, StateNodeList)

################################################################################
class StateNode(object):
    def __init__(self, range_pairs=(), minvalue=0, maxvalue=0, transcond=None, isany=False):
        self.range_pairs = range_pairs
        self.minvalue = minvalue
        self.maxvalue = maxvalue
        self.transcond = transcond
        self.isany = isany

    def __repr__(self):
        ret = 'state:'
        for r in self.range_pairs:
            ret += str(r) + ' '
        if self.isany:
            ret += 'any '
        ret = ret[:-1]
        ret += ', cond:'
        if isinstance(self.transcond, DFNode): ret += self.transcond.tocode()
        else: ret += str(self.transcond)
        return ret

class StateNodeList(object):
    def __init__(self, nodelist=()):
        self.nodelist = nodelist

    def __repr__(self):
        ret = '('
        for n in self.nodelist:
            ret += n.__repr__() + ' '
        return ret[:-1] + ')'
